/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program calculates population means or sums from a
         complete population census.

$RCSfile: census.sas,v $; 
$Revision: 1.4 $; 
$Date: 2002/02/15 21:21:26 $;   
$Author: lbednar $; 
************************************************************************/

%MACRO census(c_est_stat =,
              c_sp       =,
              c_sp_spid  =,
              c_sp_sz    =,
              c_sp_n     =,
              c_su       =,
              c_su_spid  =,
              c_su_suid  =,
              c_su_rows  =,
              c_su_cols  =,
              c_su_attr  =,
              c_unit     =,
              c_margins  = ); 

***********************************************************************;
* This macro uses the following macro variables which should          *;
* already be defined in the macro referencing environment which       *;
* invokes this macro:                                                 *;
*  EST_STAT - indicates the statistic to be estimated                 *;
*  SP       - the name of the subpop data set                         *;
*  SP_SPID  - name of subpop dset var holding subpop id               *;
*  SP_SZ    - name of subpop dset var holding subpop size             *;
*  SU       - name of dset holding sample unit observations           *;
*  SU_ROWS  - holds names of row variables                            *;
*  SU_COLS  - holds names of col variables                            *;
*  SU_ATTR  - holds name of analysis variable                         *;
*  UNIT     - indicates units used for display (1s, 10s, 100s,...)    *;
*  MARGINS  - YES indicates marginal categories will be generated     *;
***********************************************************************;

%PUT Macro CENSUS started;

%*Retrieve input DATA into working DATA sets;
   %get_sp(gs_data_in  =&c_sp,
           gs_id       =&c_sp_spid,
           gs_sz       =&c_sp_sz,
           gs_data_out =WORK.subpop);
   %get_ob(go_data_in  =&c_su,
           go_subpop   =&c_su_spid,
           go_id       =&c_su_suid,
           go_r_vars   =&c_su_rows,
           go_c_vars   =&c_su_cols,
           go_an       =&c_su_attr,
           go_an_denom =,
           go_unit     =&c_unit,
           go_data_out =WORK.obs);

*Delete previous versions of data sets used for macro processing;
PROC DATASETS LIBRARY=work;
   DELETE m_su1
          m_su2
          m_pop
          m_sp;
RUN;

*Calculate relative wt of subpopulations;
PROC SUMMARY DATA=WORK.subpop;
   VAR sz_sp;
   CLASS sp;
   OUTPUT OUT = WORK.m_sp
          SUM = sz_sp;
RUN;
DATA WORK.m_sp;
   SET WORK.m_sp;
   RETAIN sz_pop 0;
   IF _TYPE_ = 0 THEN sz_pop = sz_sp;
   ELSE IF _TYPE_ = 1 THEN
      DO;
         wt_sp = sz_sp / sz_pop;
         OUTPUT;
      END;
   DROP _TYPE_ _FREQ_;
RUN;
TITLE3 "Relative weights of subpopulations";
PROC PRINT DATA=WORK.m_sp;
   VAR sp sz_pop sz_sp wt_sp;
RUN;

********************* STEP 1 *************************;
*STEP 1 - Calculate subpop sample summary statistics;
*Sample-unit sums for all possible combinations of row
 and column variables;
PROC SORT DATA=WORK.obs;
   BY sp 
      su_id;
RUN;
PROC SUMMARY DATA=WORK.obs;
   VAR   attrib;
   BY    sp 
         su_id;
   CLASS &c_su_cols &c_su_rows;
   OUTPUT OUT=m_su   (DROP=_TYPE_ _FREQ_)
          SUM=attrib;
RUN;

PROC SORT DATA=WORK.m_su;
    BY sp 
       &c_su_cols 
       &c_su_rows;
RUN;
PROC SUMMARY DATA=WORK.m_su;
   VAR attrib;
   BY sp 
      &c_su_cols 
      &c_su_rows;
   OUTPUT OUT = m_su1
          USS = uss
          SUM = sum;
RUN;

*Replace blank values in row and column variables with "ALL";
%chg_blnk(data_in  =WORK.m_su1,
          chg_vars =&c_su_cols &c_su_rows,
          new_val  =ALL,
          data_out =WORK.m_su1)

************************ STEP 2 *****************************;
*STEP 2 - Calculate subpop contribs to population estimates;
PROC SORT DATA=WORK.m_sp;
   BY sp;
RUN;
DATA WORK.m_su2(LABEL="Subpop contribs to pop est");
   MERGE WORK.m_sp
         WORK.m_su1;
   BY sp;
   avg = sum / sz_sp;
   IF sz_sp ^> 1
       THEN v2 = 0;
   ELSE v2 = (uss/sz_sp - avg**2) / sz_sp;
   %IF &c_est_stat=MEAN %THEN
      %DO;
         est_sp   = avg;
         w_est_sp = wt_sp    * avg;
      %END;
   %ELSE %IF &c_est_stat=SUM %THEN
      %DO;
         est_sp   = sz_sp    * avg;
         w_est_sp = sz_sp    * avg;
      %END;
   w_var_sp = 0;
   LABEL uss      = "Within-subpop ucorr SS"
         sum      = "Analysis VAR subpop sum"
         avg      = "Subpop mean"
         v2       = "Subpop var"
         est_sp   = "Subpop estd &c_su_attr &c_est_stat"
         w_est_sp = "Wtd subpop est";
   DROP _type_ _freq_;
RUN;
PROC SORT DATA=WORK.m_su2;
    BY &c_su_cols 
       &c_su_rows;
RUN;
TITLE3 "Subpop contribution to population estimate";
PROC PRINT DATA=m_su2
           UNIFORM;
    VAR &c_su_cols &c_su_rows sp sz_sp avg v2 wt_sp est_sp w_est_sp;
    SUM                          sz_sp        wt_sp        w_est_sp;
    BY  &c_su_cols 
        &c_su_rows;
RUN;

************************** STEP 3 *****************************;
*STEP 3 - Calculate pop estimates by summing weighted subpop
 estimates;
PROC SORT DATA=WORK.m_su2;
   BY &c_su_rows 
      &c_su_cols;
RUN;
PROC SUMMARY DATA=WORK.m_su2;
   VAR w_est_sp w_var_sp;
   BY &c_su_rows 
      &c_su_cols;
   OUTPUT OUT = m_final ( LABEL = "Population estimates"
                          DROP  = _type_ _freq_ )
          SUM = &c_est_stat
                v_&c_est_stat;
RUN;
DATA WORK.m_final;
   SET WORK.m_final;
   sd_&c_est_stat = SQRT(v_&c_est_stat);
   LABEL &c_est_stat    = "Pop &c_su_attr &c_est_stat est"
         v_&c_est_stat  = "Estd var of pop &c_su_attr &c_est_stat est"
         sd_&c_est_stat = "Estd std of pop &c_su_attr &c_est_stat est";
RUN;

*Delete data sets used for macro processing;
PROC DATASETS LIBRARY=work;
   DELETE m_su1
          m_su2
          m_pop
          m_sp;
   *Data SET M_FINAL is preserved for passing DATA to REPOUT macro;
RUN;

%PUT Macro CENSUS ended;

%MEND census;

%*The following macro invocation is used for development testing;
%*census(c_est_stat =,
	 c_sp       =,
	 c_sp_spid  =,
	 c_sp_sz    =,
	 c_su       =,
	 c_su_spid  =,
	 c_su_suid  =,
	 c_su_rows  =,
	 c_su_cols  =,
	 c_su_attr  =,
	 c_unit     =,
	 c_margins  = );

*** END OF MACRO ***;
*** END OF MACRO ***;
