/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program calculates population estimates for each cell
         in a table using a double-sampling approach.

         An estimator is used which assumes each cell of the final
         table of estimates to represent summary statistics for ALL
         secondary sample units in the population.  Cell estimates
         represent the population estimate of a variable which takes
         zero values for secondary sample units which do not satisfy
         the conditions indicated by row and column categories in the
         table.

         At present, the program assumes three input data sets, one
         containing observations at the sample unit level, one
         containing summary statistics for strata, and one containing
         summary information about subpopulations which will contribute
         to the derived overall population estimates.

$RCSfile: double.sas,v $; 
$Revision: 1.7 $; 
$Date: 2002/02/15 21:20:28 $;   
$Author: lbednar $; 
***********************************************************************/

%Macro double( d_est_stat =,
	       d_st       =,
	       d_st_spid  =,
	       d_st_stid  =,
	       d_st_ssz   =,
	       d_st_ssz1  =,
               d_st_sz    =,
	       d_su       =,
               d_su_suid  =,
	       d_su_stid  =,
	       d_su_rows  =,
	       d_su_cols  =,
	       d_su_attr  =,
	       d_unit     =,
	       d_margins  = );

%put Macro DOUBLE started;

%LET d_est_stat = %UPCASE(&d_est_stat);
%LET d_margins  = %UPCASE(&d_margins);

%*Retrieve input data into working DATA sets;
   %get_st_s(gss_data_in  =&d_st,
             gss_sp       =&d_st_spid,
             gss_id       =&d_st_stid,
             gss_sz       =&d_st_sz,
             gss_ssz      =&d_st_ssz,
             gss_ssz1     =&d_st_ssz1,
             gss_data_out =WORK.stratum);
   %get_ob_s(gos_data_in  =&d_su,
             gos_st       =&d_su_stid,
             gos_id       =&d_su_suid,
             gos_r_vars   =&d_su_rows,
             gos_c_vars   =&d_su_cols,
             gos_an       =&d_su_attr,
             gos_unit     =&d_unit,
             gos_data_out =WORK.obs);

PROC CONTENTS DATA=WORK.stratum;
RUN;

*The program assumes existence of input DATA sets WORK.STRATUM 
 and WORK.OBS;

PROC DATASETS LIBRARY=WORK;
    DELETE m_su1
           m_su3
           m_su4
           m_su5
           m_su6
           m_sp
           m_st
           m_temp
           m_final;
RUN;

*Calculate weights of subpopulations relative to complete target
 population;
PROC SUMMARY DATA=WORK.stratum;
   VAR    sz_h;
   CLASS  sp;
   OUTPUT OUT = WORK.m_sp
          SUM = sz_sp;
RUN;
DATA WORK.m_sp       ( LABEL = "Subpop attributes" );
   SET    WORK.m_sp;
   RETAIN sz_pop 0;
   IF _TYPE_ = 0
     THEN sz_pop = sz_sp;
   ELSE IF _TYPE_=1 THEN
      DO;
         wt_sp = sz_sp / sz_pop;
         OUTPUT;
      END;
   DROP _TYPE_ _FREQ_;
RUN;

*Calculate subpop sample sizes;
PROC SORT DATA=WORK.stratum;
    BY sp 
       h;
RUN;
PROC SUMMARY DATA=WORK.stratum;
    VAR    n1_h 
           n_h;
    BY     sp;
    OUTPUT OUT = WORK.m_temp ( DROP = _TYPE_ _FREQ_ )
           SUM = n1 
                 n;
RUN;
DATA m_sp (LABEL="Subpop attributes");
   MERGE m_temp
         m_sp;
   BY    sp;
   LABEL sp     = "Subpop ID"
         sz_pop = "Pop size"
         sz_sp  = "Subpop size"
         wt_sp  = "Subpop wt"
         n1     = "Subpop prim samp size"
         n      = "Subpop secdry samp size";
RUN;
TITLE3 "Subpopulation statistics";
PROC PRINT DATA=work.m_sp;
   VAR sp sz_pop sz_sp wt_sp n1 n;
RUN;

*Calculate subpopulation stratum wts;
DATA m_st               ( LABEL = "Stratum attributes" );
   MERGE m_sp
         work.stratum;
   BY    sp;
   w_h = sz_h / sz_sp;
   IF h > .Z THEN OUTPUT;
   LABEL w_h = "Subpop stratum wt";
   KEEP  sp h n1_h w_h n_h;
RUN;
TITLE3 "Subpop stratum statistics";
PROC PRINT DATA=WORK.m_st
           uniform;
   VAR sp h n1_h w_h n_h;
   SUM      n1_h w_h n_h;
RUN;

*Calculate secondary sample summary statistics for strata;
*Match observations to subpopulations;
PROC SORT DATA=WORK.m_st;
   BY h;
RUN;
PROC SORT DATA=WORK.obs;
   BY h;
RUN;
DATA WORK.m_su     ( LABEL = "Merged stratum/su attributes" );
   MERGE WORK.obs  ( KEEP = h su_id &d_su_rows &d_su_cols attrib
                     IN   = obs_in )
         WORK.m_st ( KEEP = sp h );
   BY    h;
   IF obs_in=1 THEN OUTPUT;
RUN;

*Sum any subcategorizations of sample unit observations within
 input observations to level of specified response category variables;
PROC SORT DATA=WORK.m_su;
   BY sp 
      h 
      su_id 
      &d_su_cols 
      &d_su_rows;
RUN;
PROC SUMMARY DATA=WORK.m_su;
   VAR attrib;
   %IF &d_margins = YES %THEN
      %DO;
         *These statments result in generation of estimates for
          marginal populations;
         CLASS &d_su_cols &d_su_rows;
         BY sp 
            h 
            su_id;
      %END;
   %ELSE
      %DO;
         *This statement results in generation of estimates for
          only those response categories provided in the input
          observation data set;
         BY sp 
            h 
            su_id 
            &d_su_cols 
            &d_su_rows;
      %END;
   OUTPUT OUT = WORK.m_su (DROP=_TYPE_ _FREQ_)
          SUM = attrib;
RUN;

******************************* STEP 1 ************************************;
*STEP 1 - Calculate stratum sample sum and uncorr SS;
*Stratum-level sums for each combination of row and column response
 categories;
PROC SORT DATA=WORK.m_su;
   BY sp 
      &d_su_cols 
      &d_su_rows 
      h 
      su_id;
RUN;
PROC SUMMARY DATA=WORK.m_su;
   VAR    attrib;
   BY     sp 
          &d_su_cols 
          &d_su_rows 
          h;
   OUTPUT OUT = WORK.m_su1 ( LABEL = "Stratum samp sums"
                             DROP  = _TYPE_             )
          USS = uss_h
          SUM = sum_h;
RUN;

*Change blank values of row and col variables to "ALL";
%chg_blnk(data_in  = WORK.m_su1,
          chg_vars = &d_su_rows &d_su_cols,
          new_val  = ALL,
          data_out = WORK.m_su1)

*The following print is used during development/troubleshooting;
TITLE3 "Stratum-level sums, all resp combs";
PROC SORT DATA=WORK.m_su1;
   BY sp 
      &d_su_cols 
      &d_su_rows 
      h;
RUN;
PROC PRINT DATA=WORK.m_su1;
   VAR sp 
       &d_su_cols 
       &d_su_rows 
       h 
       uss_h 
       sum_h;
RUN;

******************************* STEP 2 *********************************;
*STEP 2 - Calculate the sample mean and sample variance for each stratum;
*Match/merge stratum sample size statistics to sums calculated in step 1
 - required to calculate stratum mean for each resp var combination;
PROC SORT DATA=WORK.m_su1;
   BY sp 
      h;
RUN;
PROC SORT DATA=WORK.m_st;
   BY sp 
      h;
RUN;
DATA WORK.m_su2       ( LABEL = "Stratum samp mean/var" );
   MERGE WORK.m_st
         WORK.m_su1;
   BY    sp 
         h;
   avg_h = sum_h / n_h;
   IF n_h > 1
      THEN s2_h = (uss_h - n_h*(avg_h)**2 )/(n_h-1);
   ELSE IF n_h = 1
      THEN s2_h = 0;
   ELSE
      DO;
         s2_h = .c;
      END;
   w_avg_h = w_h * avg_h;
   LABEL sum_h   = "Cell stratum SS"
         uss_h   = "Cell stratum uncorr SS"
         avg_h   = "Cell stratum mean"
         s2_h    = "Cell stratum sample var"
         w_avg_h = "Cell wtd stratum mean";
   *DROP _TYPE_ _FREQ_;
RUN;

*The following sort/print used for development and troubleshooting;
PROC SORT DATA=WORK.m_su2;
   BY sp 
      h;
RUN;
TITLE3 "Stratum sample mean and sample variance";
TITLE4 "DOUBLE macro step 2";
PROC PRINT DATA=WORK.m_su2;
    VAR sp &d_su_cols &d_su_rows h n1_h n_h uss_h sum_h avg_h s2_h w_h w_avg_h;
    SUM                            n1_h n_h                            w_avg_h;
    BY  sp;
RUN;


******************************** STEP 3 ***********************************;
*STEP 3 - Calculate subpopulation means - required for calculating stratum
 contributions to estimation error;
PROC SORT DATA=WORK.m_su2;
    BY sp 
       &d_su_rows 
       &d_su_cols;
RUN;
PROC SUMMARY DATA=WORK.m_su2;
    VAR w_avg_h;
    BY  sp 
        &d_su_rows 
        &d_su_cols;
    OUTPUT OUT = WORK.m_su3( LABEL = "Subpop estd mean"
                             DROP  = _TYPE_ _FREQ_)
           SUM = avg_sp;                     *subpopulation-level mean;
RUN;

*Add subpopulation values to cell/stratum info for later use;
DATA WORK.m_su3      ( LABEL = "Subpop samp mean" );
    MERGE WORK.m_sp  (KEEP  = sp sz_sp n1 )
          WORK.m_su3;
    BY    sp;
    LABEL avg_sp = "Estd subp mean";
RUN;

*The following print is used during development and troubleshooting;
TITLE3 "Subpop means for cell categories";
TITLE4 "DOUBLE macro step 3";
PROC PRINT DATA=WORK.m_su3;
    VAR sp 
        &d_su_rows 
        &d_su_cols 
        avg_sp;
RUN;

***************************** STEP 4 **************************************;
*STEP 4 - Calculate stratum contributions to subpop estimates paralleling
 Cochrans eqn 12.24;
DATA WORK.m_su4       ( LABEL = "Wtd str mean/var contribs to subp" );
    MERGE WORK.m_su3
          WORK.m_su2;
    BY    sp 
          &d_su_rows 
          &d_su_cols;
    w_est_h = w_avg_h;
    g       = (sz_sp - n1) / (sz_sp - 1);
    w_var_h =    (1/n_h  )*w_h**2 * s2_h
               - (1/sz_sp)*w_h    * s2_h
               + g*(1/n1) *w_h    * (avg_h-avg_sp)**2;
    IF "&d_est_stat"="SUM" THEN
        DO;
            w_est_h = sz_sp    * w_avg_h;
            w_var_h = sz_sp**2 * w_var_h;
        END;
    DROP avg_sp g;
RUN;

***************************** STEP 5 *************************************;
*STEP 5 - Sum stratum contributions to obtain subpop est and estd var;
PROC SUMMARY DATA=WORK.m_su4;
    *The next two statments were used in the 23May1998 version of this
     module;
    *CLASS &d_su_rows &d_su_cols;
    *BY    sp ;
    *The next statment replaced the previous two during troubleshooting
     on 12Jan1999, CLASS stmt should not have been used previously;
    BY sp 
       &d_su_rows 
       &d_su_cols;
    VAR   w_est_h 
          w_var_h;
    OUTPUT OUT = WORK.m_su5 ( LABEL = "Subp parameter est/estd var" )
           SUM = est_sp 
                 var_sp;
RUN;
DATA WORK.m_su5 ( LABEL = "Subp parameter est/estd var" );
   SET WORK.m_su5;
   LABEL est_sp = "Estd subp &d_su_attr &d_est_stat"
         var_sp = "Estd var of subp &d_su_attr &d_est_stat est";
RUN;

***************************** STEP 6 **************************************;
*STEP 6 - Calculate weighted subpop contribs to pop estimates;
TITLE3 "Weighted subpop contributions to pop estimates";
DATA WORK.m_su6      ( LABEL = "Subp contrib to pop est" );
    MERGE WORK.m_su5
          WORK.m_sp  ( KEEP = sp wt_sp );
   BY sp;
   %IF &d_est_stat=MEAN %THEN
      %DO;
          w_est_sp = wt_sp    * est_sp;
          w_var_sp = wt_sp**2 * var_sp;
      %END;
   %ELSE %IF &d_est_stat = SUM %THEN
      %DO;
          w_est_sp =  est_sp;
          w_var_sp =  var_sp;
      %END;
   LABEL w_est_sp = "Wtd estd subp &d_su_attr &d_est_stat"
         w_var_sp = "Wtd estd var of subp &d_su_attr &d_est_stat est";
RUN;

******************************* STEP 7 ***********************************;
*STEP 7 - Sum subpop contribs to get population parameter estimates;
PROC SORT DATA=WORK.m_su6;
   BY &d_su_rows 
      &d_su_cols;
RUN;
PROC SUMMARY DATA=WORK.m_su6;
   VAR w_est_sp 
       w_var_sp;
   BY  &d_su_rows 
       &d_su_cols;
   OUTPUT OUT = WORK.m_final ( LABEL = "Pop estimates"
                               DROP  = _TYPE_ _FREQ_ )
          SUM = &d_est_stat  
                v_&d_est_stat;
RUN;
DATA WORK.m_final;
   SET WORK.m_final;
   sd_&d_est_stat = SQRT(v_&d_est_stat);
   LABEL &d_est_stat    = "Pop &d_su_attr &d_est_stat est"
         v_&d_est_stat  = "Estd var of pop &d_su_attr &d_est_stat est"
         sd_&d_est_stat = "Estd std of pop &d_su_attr &d_est_stat est";
RUN;

TITLE3 "Final population estimates";
TITLE4 "DOUBLE macro step 7";
PROC PRINT DATA=WORK.m_final;
RUN;

PROC DATASETS LIBRARY=WORK;
    DELETE m_su1
           m_su2
           m_su3
           m_su4
           m_su5
           m_su6
           m_sp
           m_st
           m_temp;
    *Data SET M_FINAL is preserved for passing DATA to REPOUT module;
RUN;

%put Macro DOUBLE ended;

%MEND double;

%*The following macro invocation is used for development testing;
%*double( d_est_stat =,
	  d_st       =,
	  d_st_spid  =,
	  d_st_stid  =,
	  d_st_ssz   =,
	  d_st_ssz1  =,
          d_st_sz    =,
	  d_su       =,
          d_su_suid  =,
	  d_su_stid  =,
	  d_su_rows  =,
	  d_su_cols  =,
	  d_su_attr  =,
	  d_unit     =,
	  d_margins  = );

*** END OF MACRO ***;
*** END OF MACRO ***;
