/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program calculates population estimates for each cell
         in a table using a double-sampling approach.

         An estimator is used which assumes each cell of the final
         table of estimates to represent summary statistics for ALL
         secondary sample units in the population.  Cell estimates
         represent the population estimate of a variable which takes
         zero values for secondary sample units which do not satisfy
         the conditions indicated by row and column categories in the
         table.

         At present, the program assumes three input data sets, one
         containing observations at the sample unit level, one
         containing summary statistics for strata, and one containing
         summary information about subpopulations which will contribute
         to the derived overall population estimates.

$RCSfile: double_rs.sas,v $; 
$Revision: 1.11 $; 
$Date: 2002/02/15 21:19:56 $;   
$Author: lbednar $; 
***********************************************************************/

%Macro double_rs( drs_est_stat      =,
		  drs_st            =,
		  drs_st_spid       =,
		  drs_st_stid       =,
		  drs_st_ssz        =,
		  drs_st_ssz1       =,
		  drs_st_sz         =,
		  drs_su            =,
		  drs_su_suid       =,
		  drs_su_stid       =,
		  drs_su_rows       =,
		  drs_su_cols       =,
		  drs_su_attr       =,
                  drs_su_attr_denom =,
		  drs_unit          =,
		  drs_margins       = );

%put Macro DOUBLE_RS started;

%LET drs_est_stat = %UPCASE(&drs_est_stat);
%LET drs_margins  = %UPCASE(&drs_margins);

%*Retrieve input data into working DATA sets;
   %get_st_s(gss_data_in  =&drs_st,
             gss_sp       =&drs_st_spid,
             gss_id       =&drs_st_stid,
             gss_sz       =&drs_st_sz,
             gss_ssz      =&drs_st_ssz,
             gss_ssz1     =&drs_st_ssz1,
             gss_data_out =WORK.stratum);
   %get_ob_s(gos_data_in  =&drs_su,
             gos_st       =&drs_su_stid,
             gos_id       =&drs_su_suid,
             gos_r_vars   =&drs_su_rows,
             gos_c_vars   =&drs_su_cols,
             gos_an       =&drs_su_attr,
             gos_an_denom =&drs_su_attr_denom,
             gos_unit     =&drs_unit,
             gos_data_out =WORK.obs);

*The program assumes existence of input DATA sets WORK.STRATUM 
 and WORK.OBS;

PROC DATASETS LIBRARY=WORK;
    DELETE m_su1
           m_su3
           m_su4
           m_su5
           m_su6
           m_su_r
           m_sp
           m_st
           m_temp
           m_final;
RUN;

*Calculate weights of subpopulations relative to complete target
 population;
PROC SUMMARY DATA=WORK.stratum;
   VAR    sz_h;
   CLASS  sp;
   OUTPUT OUT = WORK.m_sp
          SUM = sz_sp;
RUN;
DATA WORK.m_sp       ( LABEL = "Subpop attributes" );
   SET    WORK.m_sp;
   RETAIN sz_pop 0;
   IF _TYPE_ = 0
     THEN sz_pop = sz_sp;
   ELSE IF _TYPE_=1 THEN
      DO;
         wt_sp = sz_sp / sz_pop;
         OUTPUT;
      END;
   DROP _TYPE_ _FREQ_;
RUN;

*Calculate subpop sample sizes;
PROC SORT DATA=WORK.stratum;
    BY sp 
       h;
RUN;
PROC SUMMARY DATA=WORK.stratum;
    VAR    n1_h 
           n_h;
    BY     sp;
    OUTPUT OUT = WORK.m_temp ( DROP = _TYPE_ _FREQ_ )
           SUM = n1 
                 n;
RUN;
DATA m_sp (LABEL="Subpop attributes");
   MERGE m_temp
         m_sp;
   BY    sp;
   LABEL sp     = "Subpop ID"
         sz_pop = "Pop size"
         sz_sp  = "Subpop size"
         wt_sp  = "Subpop wt"
         n1     = "Subpop prim samp size"
         n      = "Subpop secdry samp size";
RUN;
TITLE3 "Subpopulation statistics";
PROC PRINT DATA=WORK.m_sp;
   VAR sp sz_pop sz_sp wt_sp n1 n;
RUN;

*Calculate subpopulation stratum wts;
DATA WORK.m_st               ( LABEL = "Stratum attributes" );
   MERGE WORK.m_sp
         WORK.stratum;
   BY    sp;
   w_h = sz_h / sz_sp;
   f_h = n_h  / sz_h;
   IF h > .Z THEN OUTPUT;
   LABEL w_h = "Subpop stratum wt";
   KEEP  sp h sz_sp sz_h n n1 n1_h w_h n_h f_h;
RUN;
TITLE3 "Subpop stratum statistics";
PROC PRINT DATA=WORK.m_st
           UNIFORM;
   VAR sp h sz_h n1_h w_h n_h f_h;
   SUM           n1_h w_h n_h;
RUN;

*Calculate secondary sample summary statistics for strata;
*Match observations to subpopulations;
PROC SORT DATA=WORK.m_st;
   BY h;
RUN;
PROC SORT DATA=WORK.obs;
   BY h;
RUN;
DATA WORK.m_su     ( LABEL = "Merged stratum/su attributes" );
   MERGE WORK.obs  ( KEEP = h su_id &drs_su_rows &drs_su_cols 
                            attrib attrib2
                     IN   = obs_in )
         WORK.m_st ( KEEP = sp h );
   BY    h;
   IF obs_in = 1 THEN OUTPUT;
RUN;

*Sum any subcategorizations of sample unit observations within
 input observations to level of specified response category variables;
PROC SORT DATA=WORK.m_su;
   BY sp 
      h 
      su_id 
      &drs_su_cols 
      &drs_su_rows;
RUN;
PROC SUMMARY DATA=WORK.m_su;
   VAR attrib
       attrib2;
   %IF &drs_margins = YES %THEN
      %DO;
         *These statments result in generation of estimates for
          marginal populations;
         CLASS &drs_su_cols &drs_su_rows;
         BY sp 
            h 
            su_id;
      %END;
   %ELSE
      %DO;
         *This statement results in generation of estimates for
          only those response categories provided in the input
          observation data set;
         BY sp 
            h 
            su_id 
            &drs_su_cols 
            &drs_su_rows;
      %END;
   OUTPUT OUT = WORK.m_su (DROP=_TYPE_ _FREQ_)
          SUM = y
                x;
RUN;
DATA WORK.m_su;
   SET WORK.m_su;
   yx = y * x;
RUN;

******************************* STEP 1 ************************************;
*STEP 1 - Calculate stratum sample sum and uncorr SS;
*Stratum-level sums for each combination of row and column response
 categories;
PROC SORT DATA=WORK.m_su;
   BY sp 
      h
      &drs_su_rows 
      &drs_su_cols 
      su_id;
RUN;
PROC SUMMARY DATA=WORK.m_su;
   VAR    y
          x
          yx;
   BY     sp
          h 
          &drs_su_rows 
          &drs_su_cols; 
   OUTPUT OUT = WORK.m_su1 ( LABEL = "Stratum samp sums"
                             DROP  = _TYPE_  _FREQ_ yx_uss_h )
          USS = y_uss_h
                x_uss_h
                yx_uss_h
          SUM = y_sum_h
                x_sum_h
                yx_sum_h;
RUN;

DATA WORK.m_su1 (LABEL = "Stratum samp sums");
   SET   WORK.m_su1;
   LABEL x_uss_h  = "Uncorr var Y stratum SS"
         y_uss_h  = "Uncorr var X stratum SS"
         x_sum_h  = "Analysis var X stratum sum"
         y_sum_h  = "Analysis var Y stratum sum"
         yx_sum_h = "Analysis var YX stratum sum";
RUN;

*Change blank values of row and col variables to "ALL";
%chg_blnk(data_in  = WORK.m_su1,
          chg_vars = &drs_su_rows &drs_su_cols,
          new_val  = ALL,
          data_out = WORK.m_su1)

******************************* STEP 2 *********************************;
*STEP 2 - Calculate the sample mean and sample variance for each stratum;
*Match/merge stratum sample size statistics to sums calculated in step 1
 - required to calculate stratum mean for each resp var combination;
PROC SORT DATA=WORK.m_su1;
   BY sp 
      h;
RUN;
PROC SORT DATA=WORK.m_st;
   BY sp 
      h;
RUN;
DATA WORK.m_su2       ( LABEL = "Stratum samp mean/var" );
   MERGE WORK.m_st
         WORK.m_su1;
   BY    sp 
         h;
   y_avg_h = y_sum_h / n_h;
   x_avg_h = x_sum_h / n_h;
   IF n_h > 1 THEN
      DO;
         y_s2_h = (y_uss_h - n_h*(y_avg_h)**2 )/(n_h-1);
         x_s2_h = (x_uss_h - n_h*(x_avg_h)**2 )/(n_h-1);
      END;
   ELSE IF n_h = 1 THEN 
      DO;
         y_s2_h = 0;
         x_s2_h = 0;
      END;
   ELSE
      DO;
         y_s2_h = .c;
         x_s2_h = .c;
      END;
    r_h        = y_sum_h / x_sum_h;
    w_r_h      = w_h * r_h;
    cov_yx_h   = (yx_sum_h - n_h*y_avg_h*x_avg_h) / (n_h-1); 
    var_r_h    =   ( (1-f_h) / ( n_h * x_avg_h**2 ) )
                  *( y_s2_h  + (r_h**2)*(x_s2_h) -2*r_h*cov_yx_h);
    w_var_r_h  = w_h * var_r_h;
    w_y_avg_h  = w_h * y_avg_h;
    w_x_avg_h  = w_h * x_avg_h;
    LABEL w_var_r_h  = "Wtd cell stratum var r"
          w_x_avg_h  = "Wtd cell var X stratum mean"
          w_y_avg_h  = "Wtd cell var Y stratum mean"
          var_r_h    = "Estd var of stratum ratio est"
          x_avg_h    = "Cell var X stratum mean"
          x_sum_h    = "Cell var X stratum SS"
          x_s2_h     = "Cell var X stratum sample var"
          x_uss_h    = "Cell var X stratum uncorr SS"
          y_avg_h    = "Cell var Y stratum mean"
          y_sum_h    = "Cell var Y stratum SS"
          y_s2_h     = "Cell var Y stratum sample var"
          y_uss_h    = "Cell var Y stratum uncorr SS";
   *DROP _TYPE_ _FREQ_;
RUN;

*The following sort/print used for development and troubleshooting;
PROC SORT DATA=WORK.m_su2;
   BY sp
      &drs_su_rows
      &drs_su_cols
      h;
RUN;
TITLE3 "Stratum sample X and Y statistics";
TITLE4 "DOUBLE_RS macro step 2";
PROC PRINT DATA=WORK.m_su2;
    VAR sp &drs_su_rows &drs_su_cols h n1_h n_h y_sum_h   x_sum_h 
                                                y_uss_h   x_uss_h 
                                                y_avg_h   x_avg_h 
                                                y_s2_h    x_s2_h
                                                cov_yx_h;
    BY  sp;
RUN;
TITLE3 "Stratum sample ratio statistics";
TITLE4 "DOUBLE_RS macro step 2";
PROC PRINT DATA=WORK.m_su2;
    VAR sp &drs_su_rows &drs_su_cols h w_h r_h      var_r_h 
                                           w_r_h    w_var_r_h;
RUN;


******************************** STEP 3 ***********************************;
*STEP 3 - Calculate subpopulation means - required for calculating stratum
 contributions to estimation error;
PROC SORT DATA=WORK.m_su2;
   BY sp 
      &drs_su_rows 
      &drs_su_cols;
RUN;
PROC SUMMARY DATA=WORK.m_su2;
   VAR w_r_h
       w_x_avg_h
       w_y_avg_h;
   BY  sp 
       &drs_su_rows 
       &drs_su_cols;
   OUTPUT OUT = WORK.m_su3( LABEL = "Subpop estd mean"
                            DROP  = _TYPE_ _FREQ_)
          SUM = rs_sp
                x_avg_sp
                y_avg_sp;
RUN;

*Add subpopulation values to cell/stratum info for later use;
DATA WORK.m_su3     (LABEL = "Subpop samp mean" );
   MERGE WORK.m_sp  (KEEP  = sp sz_sp n1 )
         WORK.m_su3;
   BY    sp;
   LABEL rs_sp     = "Estd subp &drs_su_attr/&drs_su_attr_denom rom_s est"
         x_avg_sp  = "Estd subp su mean X"
         y_avg_sp  = "Estd subp su mean Y";
RUN;

TITLE3 "Subpopulation statistics";
TITLE4 "DOUBLE_RS macro - step 3";
PROC PRINT DATA=WORK.m_su3;
RUN;

***************************** STEP 4 **************************************;
*STEP 4 - Calculate stratum contributions to subpop estimates paralleling
 Cochrans eqn 12.24;
DATA WORK.m_su4       ( LABEL = "Wtd str mean/var contribs to subp" );
    MERGE WORK.m_su3
          WORK.m_su2;
    BY    sp 
          &drs_su_rows 
          &drs_su_cols;
    *Use an expanded version of the stratum estimate of variance for the ratio
     estimate instead of Cochrans S2_h term - can be viewed as equivalent
     to using var_h as a replacement for S2_h/n_h;
    temp_rs_h      = var_r_h * n_h;
    g              = (sz_sp - n1) / (sz_sp - 1);
    w_var_rs_h     =    (1/n_h  )*w_h**2 * temp_rs_h
                      - (1/sz_sp)*w_h    * temp_rs_h
                      + g*(1/n1) *w_h    * (r_h     - rs_sp    )**2;
    w_var_x_h      =    (1/n_h  )*w_h**2 * x_s2_h
                      - (1/sz_sp)*w_h    * x_s2_h
                      + g*(1/n1) *w_h    * (x_avg_h - x_avg_sp )**2;
    w_var_y_h      =    (1/n_h  )*w_h**2 * y_s2_h
                      - (1/sz_sp)*w_h    * y_s2_h
                      + g*(1/n1) *w_h    * (y_avg_h - y_avg_sp )**2;
    *The next equation parallels eqn 4.20 from Natl FIA stat band documentation 
     rev2, but not exactly since 4.20 gives the cov of population totals.  
     Since var_x and var_y here are variances of means, the equivalent to 4.20 
     must estimate the variance of a mean rather than a total.  See Cochrans 
     eqn 12.32;
    w_cov_yx_h =   (1/sz_sp)
                 * (   (sz_sp-1)*(((n1_h-1)/(n1-1))-((n_h-1)/(sz_sp-1)))
                                * w_h * cov_yx_h / n_h**2
                     + ((sz_sp-n1)/(n1-1))
                                * w_h * (y_avg_h-y_avg_sp)*(x_avg_h-x_avg_sp));
    DROP rs_sp 
         g
         temp_rs_h;
RUN;
TITLE3 "Stratum sample ratio statistics";
TITLE4 "DOUBLE_RS macro step 4";
PROC PRINT DATA=WORK.m_su4;
    VAR sp &drs_su_rows  &drs_su_cols h w_r_h       w_var_rs_h
                                        x_avg_h     y_avg_h
                                        w_var_x_h   w_var_y_h
                                        w_cov_yx_h;
    BY  sp;
RUN;


***************************** STEP 5 *************************************;
*STEP 5 - Sum stratum contributions to obtain subpop est and estd var;
PROC SORT DATA = WORK.m_su4;
   BY sp
      &drs_su_rows
      &drs_su_cols;
PROC SUMMARY DATA=WORK.m_su4;
    VAR w_r_h 
        w_var_rs_h
        w_x_avg_h
        w_y_avg_h
        w_var_x_h
        w_var_y_h
        w_cov_yx_h;
    BY  sp 
        &drs_su_rows 
        &drs_su_cols;
    OUTPUT OUT = WORK.m_su5 ( LABEL = "Subp parameter est/estd var" )
           SUM = rs_sp 
                 var_rs_sp
                 x_avg_sp
                 y_avg_sp
                 var_x_sp
                 var_y_sp
                 cov_yx_sp;
RUN;
DATA WORK.m_su5 ( LABEL = "Subp parameter est/estd var" );
   SET WORK.m_su5;
   rc_sp     = y_avg_sp / x_avg_sp;
   var_rc_sp =   ( 1 / x_avg_sp**2 )
               * ( var_y_sp + rc_sp**2 * var_x_sp - 2 * cov_yx_sp );
   LABEL rc_sp     = "Estd subp &drs_su_attr./&drs_su_attr_denom rom_c"
         var_rc_sp = "Estd subp &drs_su_attr./&drs_su_attr_denom rom_c var"
         rs_sp     = "Estd subp &drs_su_attr./&drs_su_attr_denom rom_s"
         var_rs_sp = "Estd subp &drs_su_attr./&drs_su_attr_denom rom_s var"
         x_avg_sp  = "Estd subp x su mean"
         var_x_sp  = "Estd subp x var"
         y_avg_sp  = "Estd subp y su mean"
         var_y_sp  = "Estd subp y var"
         cov_yx_sp = "Estd subpop cov yx";
RUN;

TITLE3 "Subpopulation statistics";
TITLE4 "DOUBLE_RS macro step 5";
PROC PRINT DATA=WORK.m_su5;
RUN;

***************************** STEP 6 **************************************;
*STEP 6 - Calculate weighted subpop contribs to pop estimates;
TITLE3 "Weighted subpop contributions to pop estimates";
DATA WORK.m_su6      ( LABEL = "Subp contrib to pop est" );
   MERGE WORK.m_su5
          WORK.m_sp  ( KEEP = sp wt_sp );
   BY sp;
   w_rc_sp     = wt_sp    * rc_sp;
   w_var_rc_sp = wt_sp**2 * var_rc_sp;
   w_rs_sp     = wt_sp    * rs_sp;
   w_var_rs_sp = wt_sp**2 * var_rs_sp;
   w_x_sp      = wt_sp    * x_avg_sp;
   w_var_x_sp  = wt_sp**2 * var_x_sp;
   w_y_sp      = wt_sp    * y_avg_sp;
   w_var_y_sp  = wt_sp**2 * var_y_sp;
   LABEL w_rc_sp     = "Wtd estd subp rom_c est"
         w_var_rc_sp = "Wtd estd var of subp rom_c"
         w_rs_sp     = "Wtd estd subp rom_s est"
         w_var_rs_sp = "Wtd estd var of subp rom_s";
RUN;

******************************* STEP 7 ***********************************;
*STEP 7 - Sum subpop contribs to get population parameter estimates;
PROC SORT DATA=WORK.m_su6;
   BY &drs_su_rows 
      &drs_su_cols;
RUN;
PROC SUMMARY DATA=WORK.m_su6;
   %IF &drs_est_stat = ROM_S %THEN
      %DO;
         VAR w_rs_sp 
             w_var_rs_sp;
      %END;
   %IF &drs_est_stat = ROM_C %THEN
      %DO;
         VAR w_rc_sp
             w_var_rc_sp;
      %END;
   BY  &drs_su_rows 
       &drs_su_cols;
   OUTPUT OUT = WORK.m_final ( LABEL = "Pop estimates"
                               DROP  = _TYPE_ _FREQ_ )
          SUM = &drs_est_stat  
                v_&drs_est_stat;
RUN;
DATA WORK.m_final;
   SET WORK.m_final;
   sd_&drs_est_stat = SQRT(v_&drs_est_stat);
   LABEL &drs_est_stat    = "Estd pop &drs_su_attr/&drs_su_attr_denom &drs_est_stat"
         v_&drs_est_stat  = "Estd var of pop &drs_su_attr/&drs_su_attr &drs_est_stat"
         sd_&drs_est_stat = "Estd std of pop &drs_su_attr/&drs_su_attr &drs_est_stat";
RUN;

TITLE3 "Final population estimates";
TITLE4 "DOUBLE_RS macro step 7";
PROC PRINT DATA=WORK.m_final;
   VAR &drs_su_rows &drs_su_cols &drs_est_stat v_&drs_est_stat sd_&drs_est_stat;
RUN;

PROC DATASETS LIBRARY=work;
    DELETE m_su1
           m_su2
           m_su3
           m_su4
           m_su5
           m_su6
           m_su_r
           m_sp
           m_st
           m_temp;
    *Data SET M_FINAL is preserved for passing DATA to REPOUT module;
RUN;

%put Macro DOUBLE_RS ended;

%MEND double_rs;

%*The following macro invocation is used for development testing;
%*double_rs( drs_est_stat =,
	     drs_st       =,
	     drs_st_spid  =,
	     drs_st_stid  =,
	     drs_st_ssz   =,
	     drs_st_ssz1  =,
	     drs_st_sz    =,
	     drs_su       =,
	     drs_su_suid  =,
	     drs_su_stid  =,
	     drs_su_rows  =,
	     drs_su_cols  =,
	     drs_su_attr  =,
	     drs_unit     =,
	     drs_margins  = );

*** END OF MACRO ***;
*** END OF MACRO ***;
