/****************************** HEADER *********************************
 Larry Bednar, Bednar Consulting, 3226 NE 24th Ave, Portland, OR, 97212
 503-493-8542
 PURPOSE: As a component of the TABLING application, this SAS program
          reads a SAS data set specified to contain sample unit
          observations and outputs that data to the SAS data set used
          within the TABLING application to hold observation data for
          population estimates assuming stratified sampling.

          Most of the macro code here is designed to change numeric row
          and column variables to character variables with the same
          name and labelling in the output data set.

$RCSfile: get_ob.sas,v $; 
$Revision: 1.8 $; 
$Date: 2002/01/31 19:35:36 $;   
$Author: lbednar $; 
************************************************************************
/******************************* CODE **********************************/

%MACRO get_ob(go_data_in  =,
              go_subpop   =,
              go_id       =,
              go_r_vars   =,
              go_c_vars   =,
              go_an       =,
              go_an_denom =,
              go_unit     =1,
              go_data_out =WORK.obs);

************************************************************************;
* This macro requires user specification of the following variables    *;
* at invocation:                                                       *;
*  GO_DATA_IN - indicates the SAS two-part data set specification for  *;
*            the data set containing the incoming data                 *;
*  GO_SUBPOP  - indicates the name of the SAS variable holding the     *;
*            subpop ID variable                                        *;
*  GO_ID      - indicates the name of the SAS variable holding the     *;
*            stratum ID variable                                       *;
*  GO_R_VARS  - indicates the list of variables used for row           *;
*            classifications in the desired output table               *;
*  GO_C_VARS  - indicates the list of variables used to for column     *;
*            classifications in the desired output table               *;
*  GO_AN      - indicates the name of the SAS variable holding the     *;
*            sample statistic for which population estimates are       *;
*            desired                                                   *;
*  GO_AN_DENOM - name of 2d SAS variable used as the denominator of a  *;
*             specified ratio statistic                                *;
*  GO_UNIT    - unit in which final figures are reported               *;
*            (10s, 1000s,...)                                          *;
*  GO_DATA_OUT- indicates the name of the SAS data set to which data   *;
*            will be written                                           *;
********************************************************************** *;

%PUT "TABLING Module GET_OB started";

%*Specify variables which are to be used only in the local referencing
  environment;

   %LOCAL go_lib 
          go_dsn                       /*input library and data set */
          go_i 
          go_j                         /*loop variables             */
          go_row1 
          go_row2 
          go_row3 
          go_row4                      /*specified row variables    */
          go_col1 
          go_col2 
          go_col3 
          go_col4                      /*specified col variables    */
          go_fmt1 
          go_fmt2 
          go_fmt3 
          go_fmt4 
          go_fmt5                      /*vars containing variable   */
                                       /*  formatting statements    */
          go_rentxt1 
          go_rentxt2 
          go_rentxt3
          go_rentxt4 
          go_rentxt5 
          go_rentxt6
          go_rentxt7 
          go_rentxt8                   /*stmts to rename vars       */
          go_asntxt1 
          go_asntxt2 
          go_asntxt3
          go_asntxt4 
          go_asntxt5 
          go_asntxt6
          go_asntxt7 
          go_asntxt8                   /*stmts to reassign var vals */
          go_lastrow 
          go_lastcol                   /*nmbr of rows and cols      */
          go_selrow                    /*list of row and col vars   */
          go_label1 
          go_label2 
          go_label3 
          go_label4                    /*stmts to label vars        */
          go_label5 
          go_label6 
          go_label7 
          go_label8
          go_fmtstmt                   /*txt for DATA FORMAT stmt   */
          go_renstmt                   /*txt for RENAME data set    */
                                       /*  option                   */
          go_lblstmt                   /*txt for LABEL stmt         */
          go_asnstmt;                  /*txt for var assignments    */

%*Separate SAS library and data set names from input specification;

   %IF %INDEX(&go_data_in,.) > 0 %THEN
      %DO;
         %LET go_lib = %SCAN(&go_data_in, 1);
         %LET go_dsn = %SCAN(&go_data_in, 2);
      %END;
   %ELSE
      %DO;
         %LET go_lib = WORK;
         %LET go_dsn = &go_data_in;
      %END;

   %*PUT go_lib = &go_lib;
   %*PUT go_dsn = &go_dsn;

%*Determine how many row variables have been specified and create SAS
   statements needed to rename and reassign values of those variables in
   a later DATA step;

    %LET go_i = 1;
    %DO %WHILE( &go_i < 5 );
       %LET go_row&go_i = %SCAN(&go_r_vars, &go_i);
       %IF &&&go_row&go_i ^=  %THEN
          %DO;
             %LET go_lastrow  = %EVAL(&go_i);
             %LET go_rentxt&go_i = %STR( &&&go_row&go_i=r&go_i);
             %LET go_asntxt&go_i = %STR( &&&go_rentxt&go_i;);
          %END;
       %ELSE
          %DO;
             %LET go_asntxt&go_i = ;
             %LET go_rentxt&go_i = ;
          %END;
       %LET go_i = %EVAL(&go_i+1);
    %END;

    %*PUT lastrow = &go_lastrow;

%*Determine how many col variables have been specified and create SAS
  statements needed to rename and reassign values of those variables in
  a later DATA step;

   %LET go_j = 1;
   %DO %WHILE( &go_j < 5 );
      %LET go_col&go_j = %SCAN(&go_c_vars, &go_j);
      %IF &&&go_col&go_j ^=  %THEN
         %DO;
            %LET go_lastcol     = %EVAL(&go_j);
            %LET go_rentxt&go_i = %STR( &&&go_col&go_j=c&go_j);
            %LET go_asntxt&go_i = %STR( &&&go_rentxt&go_i;);
         %END;
      %ELSE
         %DO;
            %LET go_asntxt&go_i = ;
            %LET go_rentxt&go_i = ;
         %END;
      %LET go_j = %EVAL(&go_j+1);
      %LET go_i = %EVAL(&go_i+1);
   %END;

   %*PUT lastcol = &go_lastcol;

%*Create text for SQL "IN (a,b,c)" statement used to collect
  information about SAS variables specified for row and column
  classifications;

   %LET go_selrow   = ;
   %LET go_firstval = YES;
   %LET go_i        = 1;
   %DO %WHILE( &go_i <= &go_lastrow );
      %IF &go_firstval = NO %THEN %LET go_selrow = %STR(&go_selrow,);
      %LET go_firstval = NO;
      %LET go_selrow   = %STR(&go_selrow%'&&&go_row&go_i%');
      %LET go_i        = %EVAL(&go_i+1);
   %END;

   %LET go_i=1;
   %DO %WHILE( &go_i <= &go_lastcol );
      %IF &go_firstval = NO %THEN %LET go_selrow = %STR(&go_selrow,);
      %LET go_firstval = NO;
      %LET go_selrow   = %STR(&go_selrow%'&&&go_col&go_i%');
      %LET go_i        = %EVAL(&go_i+1);
   %END;
   %LET go_selrow = %UPCASE(&go_selrow);

   %*PUT SELROW = &go_selrow;


*Create table listing variable attributes of row and column variables;

   PROC SQL;

      CREATE TABLE WORK.m_vars AS
      SELECT name,
             length,
             TRIM(LEFT(name)) || "='" || TRIM(LEFT(label)) ||"'" as ltxt
      FROM   dictionary.columns
      WHERE  UPCASE(libname) = "%UPCASE(&go_lib)"  and
             UPCASE(memname) = "%UPCASE(&go_dsn)"  and
             UPCASE(name) IN (&go_selrow);

   QUIT;

*Assign text strings needed for FORMAT statement in final DATA
 step to macro variables;

  %LET go_i = 1;
  %DO %WHILE( &go_i < 9 );
     %LET go_fmt&go_i   = ;
     %LET go_label&go_i = ;
     %LET go_i          = %EVAL(&go_i+1);
  %END;

  DATA WORK.m_txt;
     FORMAT len $8.;
     SET work.m_vars;
     len   = MAX(3,length);
     ftxt  = COMPRESS(name) || " $" || COMPRESS(len) || ".";
     fname = "go_fmt"   || compress(_N_);
     lname = "go_label" || compress(_N_);
     CALL SYMPUT(COMPRESS(fname),ftxt);
     CALL SYMPUT(COMPRESS(lname),ltxt);
  RUN;

  %LET go_fmtstmt = ;
  %LET go_lblstmt = ;
  %LET go_asnstmt = ;
  %LET go_renstmt = ;
  %LET go_i = 1;
  %DO %WHILE( &go_i < 9 );
     %LET go_fmtstmt = &go_fmtstmt &&&go_fmt&go_i;
     %LET go_lblstmt = &go_lblstmt &&&go_label&go_i;
     %IF &&&go_asntxt&go_i ^=  %THEN %LET go_asnstmt = %STR(&go_asnstmt &&&go_asntxt&go_i;);
     %LET go_renstmt = &go_renstmt &&&go_rentxt&go_i;
     %LET go_i       = %EVAL(&go_i+1);
  %END;

*Create the observation data set used by the tabling application, rename
 variables to names expected by the application, convert all numeric
 variables in original data set to character variables;

   DATA &go_data_out(LABEL="Input observation data");
      FORMAT &go_fmtstmt;
      %IF &go_an_denom =  %THEN 
         %DO;
            SET &go_data_in (RENAME=(&go_subpop = sp
                                     &go_id     = su_id
                                     &go_an     = attrib
                                     &go_renstmt));
         %END;
      %ELSE
         %DO;
            SET &go_data_in (RENAME=(&go_subpop   = sp
                                     &go_id       = su_id
                                     &go_an       = attrib
                                     &go_an_denom = attrib2
                                     &go_renstmt));
         %END;
      *Relabel subpop ID, samp unit ID, and attribute variable names
       for internal program consistency -
       leave labelling of row, col variables intact for
       interpretability;
      attrib = attrib/&go_unit;
      %IF &go_an_denom =  %THEN 
         %DO;
            attrib2 = attrib2 / &go_unit;
         %END;
      &go_asnstmt
      LABEL sp    = "subpop  id"
            su_id = "sample unit id"
            &go_lblstmt;
      %IF &go_an_denom =  %THEN 
         %DO;
            KEEP sp
                 su_id
                 &go_r_vars
                 &go_c_vars
                 attrib;
         %END;
      %ELSE
         %DO;
            KEEP sp
                 su_id
                 &go_r_vars
                 &go_c_vars
                 attrib
                 attrib2;
         %END;
   RUN;

   *Display characteristics of created data set;
   PROC CONTENTS DATA = &go_data_out;
   RUN;

   *Delete working data sets created during this macro;
   PROC DATASETS LIBRARY=WORK;
      DELETE m_vars 
             m_txt;
   RUN;

%MEND get_ob;

*Macro invocation used only during code development and troubleshooting;
%*get_ob(go_data_in  =work.one,
        go_subpop   =st,
        go_id       =id,
        go_r_vars   =rowv1 rowv2,
        go_c_vars   =colv1 colv2,
        go_an       =varin,
        go_an_denom =,
        go_unit     =1,
        go_data_out =WORK.obs);


*** END OF MACRO ***;
*** END OF MACRO ***;
