/****************************** HEADER *********************************
 Larry Bednar, Bendar Consulting, 3226 NE 24th Ave, Portland, OR 97212
 503-493-8542
 PURPOSE: As a component of the TABLING application, this SAS program
          reads a SAS data set specified to contain sample unit
          observations and outputs that data to the SAS data set used
          within the TABLING application to hold observation data for
          population estimates assuming stratified sampling.

          Most of the macro code here is designed to change numeric row
          and column variables to character variables with the same
          name and labelling in the output data set.

$RCSfile: get_ob_s.sas,v $; 
$Revision: 1.6 $; 
$Date: 2002/01/31 20:05:16 $;   
$Author: lbednar $; 

************************************************************************
/******************************* CODE **********************************/

%MACRO get_ob_s(gos_data_in  =,
                gos_st       =,
                gos_id       =,
                gos_r_vars   =,
                gos_c_vars   =,
                gos_an       =,
                gos_an_denom =,
                gos_unit     =,
                gos_data_out =WORK.obs);

***********************************************************************
* This macro requires user specification of the following variables   *
* at invocation:                                                      *
*  DATA_IN - indicates the SAS two-part data set specification for    *;
*            the data set containing the incoming data                *;
*  ST      - indicates the name of the SAS variable holding the       *;
*            stratum ID variable                                      *;
*  ID      - indicates the name of the SAS variable holding the       *;
*            stratum ID variable                                      *;
*  R_VARS  - indicates the list of variables used for row             *;
*            classifications in the desired output table              *;
*  C_VARS  - indicates the list of variables used to for column       *;
*            classifications in the desired output table              *;
*  AN      - indicates the name of the SAS variable holding the       *;
*            sample statistic for which population estimates are      *;
*            desired                                                  *;
*  AN_DENOM- indicates the name of the SAS variable holding the       *;
*            sample statistic to be used for the denominator of a     *;
*            ratio estimate                                           *;
*  UNIT    - unit in which final figures are reported (10s, 1000s,...)*;
*  DATA_OUT- indicates the name of the SAS data set to which data     *;
*            will be written                                          *;
***********************************************************************;

%PUT GET_OB_S started;

%*Specify variables which are to be used only in the local referencing
  environment;

   %LOCAL gos_lib 
          gos_dsn                  /*input library and data set */

          gos_i 
          gos_j                    /*loop variables             */

          gos_row1 
          gos_row2 
          gos_row3 
          gos_row4                 /*specified row variables    */

          gos_col1 
          gos_col2 
          gos_col3 
          gos_col4                 /*specified col variables    */

          gos_fmt1 
          gos_fmt2 
          gos_fmt3 
          gos_fmt4 
          gos_fmt5                 /*vars containing variable   */
                                   /*  formatting statements    */
          gos_rentxt1 
          gos_rentxt2 
          gos_rentxt3
          gos_rentxt4 
          gos_rentxt5 
          gos_rentxt6
          gos_rentxt7 
          gos_rentxt8              /*stmts to rename vars       */

          gos_asntxt1 
          gos_asntxt2 
          gos_asntxt3
          gos_asntxt4 
          gos_asntxt5 
          gos_asntxt6
          gos_asntxt7 
          gos_asntxt8              /*stmts to reassign var vals */

          gos_lastrow 
          gos_lastcol              /*nmbr of rows and cols      */

          gos_selrow               /*list of row and col vars   */

          gos_label1 
          gos_label2 
          gos_label3 
          gos_label4
          gos_label5 
          gos_label6 
          gos_label7 
          gos_label8               /*stmts to label vars        */

          gos_fmtstmt              /*txt for DATA FORMAT stmt   */

          gos_renstmt              /*txt for RENAME data set    */
                                   /*  option                   */
          gos_lblstmt              /*txt for LABEL stmt         */
          gos_asnstmt;             /*txt for var assignments    */

%*Separate SAS library and data set names from input specification;

%PUT "LIBRARY/DSN section started";

   %IF %INDEX(&gos_data_in,.) > 0 %THEN
      %DO;
         %LET gos_lib = %SCAN(&gos_data_in,1);
         %LET gos_dsn = %SCAN(&gos_data_in,2);
      %END;
   %ELSE
      %DO;
         %LET gos_lib = WORK;
         %LET gos_dsn = &gos_data_in;
      %END;

   %PUT lib = &gos_lib;
   %PUT dsn = &gos_dsn;

%*Determine how many row variables have been specified and create SAS
   statements needed to rename and reassign values of those variables in
   a later DATA step;

%PUT "Row variable section started";

    %LET gos_i = 1;
    %DO %WHILE(&gos_i<5);
       %LET gos_row&gos_i = %SCAN(&gos_r_vars,&gos_i);
       %IF &&&gos_row&gos_i^=  %THEN
          %DO;
             %LET gos_lastrow      = %EVAL(&gos_i);
             %LET gos_rentxt&gos_i = %STR( &&&gos_row&gos_i = r&gos_i);
             %LET gos_asntxt&gos_i = %STR( &&&gos_rentxt&gos_i;);
          %END;
       %ELSE
          %DO;
             %LET gos_asntxt&gos_i = ;
             %LET gos_rentxt&gos_i = ;
          %END;
       %LET gos_i = %EVAL(&gos_i+1);
    %END;

    %PUT lastrow = &gos_lastrow;

%*Determine how many col variables have been specified and create SAS
  statements needed to rename and reassign values of those variables in
  a later DATA step;

%PUT "Col variable section started";

   %LET gos_j = 1;
   %DO %WHILE( &gos_j < 5 );
      %LET gos_col&gos_j = %SCAN(&gos_c_vars,&gos_j);
      %IF &&&gos_col&gos_j^=  %THEN
         %DO;
            %LET gos_lastcol      = %EVAL(&gos_j);
            %LET gos_rentxt&gos_i = %STR( &&&gos_col&gos_j = c&gos_j);
            %LET gos_asntxt&gos_i = %STR( &&&gos_rentxt&gos_i;);
         %END;
      %ELSE
         %DO;
            %LET gos_asntxt&gos_i = ;
            %LET gos_rentxt&gos_i = ;
         %END;
      %LET gos_j = %EVAL(&gos_j+1);
      %LET gos_i = %EVAL(&gos_i+1);
   %END;

   %PUT lastcol = &gos_lastcol;

%*Create text for SQL "IN (a,b,c)" statement used to collect
  information about SAS variables specified for row and column
  classifications;

%PUT "SQL IN stmt section started";

   %LET gos_selrow   = ;
   %LET gos_firstval = YES;
   %LET gos_i        = 1;
   %DO %WHILE( &gos_i <= &gos_lastrow );
      %IF &gos_firstval = NO %THEN %LET gos_selrow = %STR(&gos_selrow,);
      %LET gos_firstval = NO;
      %LET gos_selrow   = %STR(&gos_selrow%'&&&gos_row&gos_i%');
      %LET gos_i        = %EVAL(&gos_i+1);
   %END;

   %LET gos_i = 1;
   %DO %WHILE( &gos_i <= &gos_lastcol );
      %IF &gos_firstval = NO %THEN %LET gos_selrow = %STR(&gos_selrow,);
      %LET gos_firstval = NO;
      %LET gos_selrow   = %STR(&gos_selrow%'&&&gos_col&gos_i%');
      %LET gos_i        = %EVAL(&gos_i+1);
   %END;
   %LET gos_selrow = %UPCASE(&gos_selrow);

   %PUT SELROW = &gos_selrow;


*Create table listing variable attributes of row and column variables;

%PUT "Variable attr section started";

   PROC SQL;

      CREATE TABLE work.m_vars AS
      SELECT name,
             length,
             TRIM(LEFT(name)) || "='" || TRIM(LEFT(label)) ||"'" as ltxt
      FROM   dictionary.columns
      WHERE  UPCASE(libname) = "%UPCASE(&gos_lib)"
        AND  UPCASE(memname) = "%UPCASE(&gos_dsn)"
        AND  UPCASE(name)   IN (&gos_selrow);

   QUIT;

*Assign text strings needed for FORMAT statement in final DATA
 step to macro variables;

%PUT "Format stmt section started";

  %LET gos_i = 1;
  %DO %WHILE( &gos_i < 9 );
     %LET gos_fmt&gos_i   = ;
     %LET gos_label&gos_i = ;
     %LET gos_i           = %EVAL(&gos_i+1);
  %END;

  DATA m_txt;
     FORMAT len $8.;
     SET work.m_vars;
     len   = MAX(3,length);
     ftxt  = COMPRESS(name) || " $" || COMPRESS(len) || ".";
     fname = "gos_fmt"   || COMPRESS(_N_);
     lname = "gos_label" || COMPRESS(_N_);
     CALL SYMPUT(COMPRESS(fname),ftxt);
     CALL SYMPUT(COMPRESS(lname),ltxt);
  RUN;

  %LET gos_fmtstmt = ;
  %LET gos_lblstmt = ;
  %LET gos_asnstmt = ;
  %LET gos_renstmt = ;
  %LET gos_i = 1;
  %DO %WHILE( &gos_i < 9 );
     %LET gos_fmtstmt = &gos_fmtstmt &&&gos_fmt&gos_i;
     %LET gos_lblstmt = &gos_lblstmt &&&gos_label&gos_i;
     %IF &&&gos_asntxt&gos_i^= %THEN %LET gos_asnstmt = %STR(&gos_asnstmt &&&gos_asntxt&gos_i;);
     %LET gos_renstmt = &gos_renstmt &&&gos_rentxt&gos_i;
     %LET gos_i       = %EVAL(&gos_i+1);
  %END;

*Create the observation data set used by the tabling application, rename
 variables to names expected by the application, convert all numeric
 variables in original data set to character variables;

%PUT "DATA extraction section started";

   DATA &gos_data_out(LABEL = "Input observation data");
      FORMAT &gos_fmtstmt;
      %IF &gos_an_denom =  %THEN 
         %DO;
	    SET &gos_data_in(RENAME = (&gos_st = h
				       &gos_id = su_id
				       &gos_an = attrib
				       &gos_renstmt));
         %END;
      %ELSE
         %DO;
	    SET &gos_data_in(RENAME = (&gos_st       = h
				       &gos_id       = su_id
				       &gos_an       = attrib
                                       &gos_an_denom = attrib2
				       &gos_renstmt));
         %END;
      *Relabel stratum ID, samp unit ID, and attribute variable names
       for internal program consistency -
       leave labelling of row, col variables intact for
       interpretability;
      attrib = attrib / &gos_unit;
      %IF &gos_an_denom =  %THEN 
         %DO;
            attrib2 = attrib2 / &gos_unit;
         %END;
      &gos_asnstmt
      LABEL h     ="stratum id"
            su_id ="sample unit id"
            &gos_lblstmt;
      %IF &gos_an_denom =  %THEN 
         %DO;
	    KEEP h
		 su_id
		 &gos_r_vars
		 &gos_c_vars
		 attrib;
         %END;
      %ELSE
         %DO;
	    KEEP h
		 su_id
		 &gos_r_vars
		 &gos_c_vars
		 attrib
                 attrib2;
         %END;
   RUN;

   PROC CONTENTS DATA=&gos_data_out;
   RUN;

   *Delete working data sets created during this macro;
   PROC DATASETS LIBRARY = WORK;
      DELETE m_vars 
             m_txt;
   RUN;

%MEND get_ob_s;

*The following macro invocation is used for development
 troubleshooting;
%*get_ob_s(gos_data_in =test.test_obs,
          gos_st       =str_id,
          gos_id       =obs_id,
          gos_r_vars   =frow srow,
          gos_c_vars   =fcol,
          gos_an       =fakevol,
          gos_an_denom =,
          gos_unit     =1,
          gos_data_out = WORK.obs);


*** END OF MACRO ***;
*** END OF MACRO ***;
