/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: In the TABLING application, this SAS program defines the macro
         responsible for checking job specifications provided by the
         user to ensure that no gross specification errors are present;

$RCSfile: in_chk.sas,v $; 
$Revision: 1.11 $; 
$Date: 2002/02/06 21:16:17 $;   
$Author: lbednar $; 
************************************************************************/

%*This utility macro is used within the IN_CHK macro to determine
  whether a macro variable has been assigned a value;
%MACRO symchk(sc_name);
   %*Passed value should be the name of a macro variable to be tested;
   %IF &&&sc_name ^=       %THEN
      %LET sc_value = YES;
   %ELSE
      %LET sc_value = NO;
   &sc_value
%MEND symchk;

%*This macro is used to set the macro response;
%MACRO set_response (sr_var, sr_resp);
   %*NOTE - sr_var value is a macro variable name, sr_resp value is a string;
   %LET sr_resp = %UPCASE(&sr_resp);
   %IF &sr_resp = ERROR   %THEN %LET &sr_var = &sr_resp;
   %IF &sr_resp = WARNING %THEN
      %DO;
         %*WARNING never overwrites a preexisting ERROR response;
         %IF &&&sr_var = ERROR %THEN %LET &sr_var = &&&sr_var;
         %ELSE                       %LET &sr_var = &sr_resp;
      %END;
%MEND set_response;

%*This macro is used within the IN_CHK macro to verify that specified
  variables are of a specific data type;
%MACRO chktype(ct_dset, ct_var_in, ct_vtype, ct_resp_var);

   %LOCAL ct_correct_vtype;

   %IF %INDEX(&ct_dset,.)>0 %THEN 
      %DO;
         %LET ct_lib  = %SCAN(&ct_dset,1,.);
         %*PUT "CT_LIB  value is %UPCASE(&ct_lib)";
         %LET ct_dset = %SCAN(&ct_dset,2,.);
         %*PUT "CT_DSET value is %UPCASE(&ct_dset)";
      %END;
   %ELSE %LET ct_lib  = WORK;

   %LET ct_vtype = %UPCASE(&ct_vtype);

   %IF &ct_vtype ^= CHARACTER AND
       &ct_vtype ^= NUMBER        %THEN
      %DO;
         %PUT "Macro CHKTYPE: 3d parameter (&ct_vtype) interpreted as (NUMBER)";
         %LET ct_vtype = NUMBER;
      %END;

   %LET ct_correct_vtype = NO;

   PROC CONTENTS
      DATA = &ct_lib..&ct_dset
      OUT  = work.m_dvars (KEEP=name type)
      NOPRINT;
   RUN;
   DATA work.m_dvars2;
      SET work.m_dvars;
      name_in = "&ct_var_in";
      IF UPCASE(name)=UPCASE("&ct_var_in") THEN
         DO;
            SELECT ("&ct_vtype");
               WHEN ("NUMBER")     IF type=1 THEN CALL SYMPUT('ct_correct_vtype','YES');
               WHEN ("CHARACTER")  IF type=1 THEN CALL SYMPUT('ct_correct_vtype','YES');
               OTHERWISE;
            END;
            STOP;
         END;
   PROC DATASETS LIBRARY=work;
      DELETE m_dvars 
             m_dvars2;
   RUN;

   %IF &ct_correct_vtype = NO %THEN
      %DO;
         %set_response(&ct_resp_var,ERROR);
         %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
         %PUT "   var %UPCASE(&ct_var_in) must be a %UPCASE(&ct_vtype) var";
      %END;

%MEND chktype;

%*This macro is used within the IN_CHK macro to verify that specified
  variables are members of the data sets specified;
%MACRO chkvar(cv_par, cv_dset, cv_var_in, cv_need, cv_err_resp_var, cv_presence_resp_var);

   %* cv_par               = parameter to be tested
      cv_dset              = data set in which value should be located
      cv_var_in            = parameter value
      cv_need              = required/not required status of variable
      cv_err_resp_var      = variable in which ERROR status will be updated
      cv_presence_resp_var = variable used to indicate presence/absence of variable 
                             from specified data set;

   %LOCAL cv_lib
          cv_var_present;

   %LET cv_need = %UPCASE(&cv_need);
   %LET cv_par  = %UPCASE(&cv_par);

   %IF "&cv_need" = "REQUIRED" %THEN
      %DO;
         %IF %symchk(cv_var_in) = YES %THEN
            %DO;

               %IF %INDEX(&cv_dset,.)>0 %THEN 
                  %DO;
                     %LET cv_lib  = %SCAN(&cv_dset,1,.);
                     %LET cv_dset = %SCAN(&cv_dset,2,.);
                  %END;
               %ELSE %LET cv_lib   = WORK;

               PROC CONTENTS
                  DATA = &cv_lib..&cv_dset
                  OUT  = work.m_dvars (KEEP=name type)
                  NOPRINT;
               RUN;

               %LET cv_var_present = NO;

               DATA _NULL_;
                  SET work.m_dvars;
                  IF UPCASE(name)=UPCASE("&cv_var_in") THEN
                        CALL SYMPUT('cv_var_present','YES');
               RUN;

               %IF &cv_var_present=NO %THEN
                  %DO;
                     %set_response(&cv_err_resp_var,ERROR);
                     %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
                     %PUT "   &cv_par var %UPCASE(&cv_var_in) not found on data set %UPCASE(&cv_dset)";
                  %END;

               PROC DATASETS LIBRARY=work;
                  DELETE m_dvars;
               RUN;

            %END;
         %ELSE
            %DO;
               %LET cv_var_present = UNK;
               %set_response(&cv_err_resp_var,ERROR);
               %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
               %PUT "   parameter &cv_par must specify a variable";                
            %END;
      %END;

   %ELSE
      %DO;
         %IF "&cv_need" ^= "UNUSED" %THEN
            %PUT "Macro CHKVAR: parameter &cv_par interpreted as UNUSED";
         %IF %symchk(cv_var_in)=YES %THEN
            %DO;
               %LET cv_var_present = UNK;
               %set_response(&cv_err_resp_var,WARNING);
               %PUT "TABLING PGM JOB SPECIFICATION WARNING:";
               %PUT "   parameter &cv_par value is not required";
            %END;
      %END;


   %LET &cv_presence_resp_var = &cv_var_present;

%MEND chkvar;

%MACRO dset_exists(de_dset, de_err_resp_var, de_resp);
%*This macro is used used within the IN_CHK macro to test for the existance
  of the user-specified SAS data sets. The macro sets the variable macro
  variable specified in "de_resp_var" to YES if data set indicated by "de_dset"
  exists, NO otherwise;

   %LOCAL de_dset_exists;

   %IF &de_dset ^=  %THEN
      %DO;

         DATA _NULL_;
            IF 0 THEN SET &de_dset;
            STOP;
         RUN;

         %IF &SYSERR=0 %THEN %LET de_dset_exists = YES;
         %ELSE               %LET de_dset_exists = NO;

         %IF de_dset_exists = NO %THEN
            %DO;
               %set_response(de_err_resp_var,ERROR);
               %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
               %PUT "   data set %UPCASE(&de_dset) does not exist";
            %END;

         %LET &de_resp = &de_dset_exists;

      %END;

%MEND dset_exists;

%MACRO in_chk(ic_title   =,
              ic_title2  =,
              ic_method  =,
              ic_est_stat=,
              ic_sp      =,
              ic_sp_spid =,
              ic_sp_sz   =,
              ic_sp_ssz  =,
              ic_st      =,
              ic_st_spid =,
              ic_st_stid =,
              ic_st_sz   =,
              ic_st_ssz1 =,
              ic_st_ssz  =,
              ic_su      =,
              ic_su_spid =,
              ic_su_stid =,
              ic_su_suid =,
              ic_su_rows =,
              ic_su_cols =,
              ic_su_attr =,
              ic_unit    =,
              ic_sig     =,
              ic_margins =,
              ic_print   =,
              ic_libout  =,
              ic_dataout =,
              ic_htmlout =,
              ic_var_out =);

   ***********************************************************************;
   * This macro assumes nothing about the referencing environment calling ;
   * this macro.  All output is provided by returned text or SAS LOG      ;
   * output.  All input to this macro is provided by explicity passing of ;
   * macro variable values at macro invocation.                           ;
   *                                                                      ;
   * The overall finding of the IN_CHK macro (i.e. ERROR, WARNING, OK)    ;
   * is passed to the calling program via the macro variable specified by ;
   * the value of the IC_VAR_OUT  invcation variable                      ;
   ***********************************************************************;

   %LOCAL ic_corrtype
          ic_present
          ic_response
          ic_var1
          ic_var2;

   %LET ic_method   = %UPCASE(&ic_method);
   %LET ic_est_stat = %UPCASE(&ic_est_stat);
   %LET ic_margins  = %UPCASE(&ic_margins);
   %LET ic_print    = %UPCASE(&ic_print);

   %*Define variables used to pass specification checking results from 
     subsidiary utility macros;
   %LET ic_corrtype    = NO;
   %LET ic_present     = NO;
   %LET ic_dset_exists = NO;
   %LET ic_response    = OK;

   %IF &ic_method ^= DOUBLE AND
       &ic_method ^= STRAT  AND
       &ic_method ^= RANDOM AND
       &ic_method ^= CENSUS      %THEN
      %DO;
         %set_response(ic_response,ERROR);
         %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
         %PUT "   specification for METHOD must be DOUBLE, STRAT, RANDOM or CENSUS";
         %PUT "   specified value is &ic_method";
      %END;

   %IF (&ic_method    = DOUBLE  OR
        &ic_method    = STRAT   OR
        &ic_method    = RANDOM      ) AND
       (&ic_est_stat ^= MEAN    AND
        &ic_est_stat ^= SUM     AND
        &ic_est_stat ^= ROM_S   AND
        &ic_est_stat ^= ROM_C       )       %THEN
      %DO;
         %set_response(ic_response,ERROR);
         %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
         %PUT "   EST_STAT must have value MEAN, SUM, ROM_S, or ROM_C for specified METHOD";
      %END;

   %IF (&ic_method    = CENSUS       ) AND
       (&ic_est_stat ^= MEAN    AND
        &ic_est_stat ^= SUM          ) %THEN
      %DO;
         %set_response(ic_response,ERROR);
         %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
         %PUT "   EST_STAT must have value MEAN or SUM for CENSUS method";
      %END;

   %IF (&ic_est_stat            = ROM_S OR
        &ic_est_stat            = ROM_C     ) AND
        %SCAN(&ic_su_attr,2,/)  =                   %THEN
      %DO;
         %set_response(ic_response,ERROR);
         %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
         %PUT "   ROM estimates require specification of two variables in SU_ATTR value";
      %END;
 
   %IF  &ic_est_stat           ^= ROM_S AND 
        &ic_est_stat           ^= ROM_C AND
        %SCAN(&ic_su_attr,2,/) ^=           %THEN
      %DO;
         %set_response(ic_response,WARNING);
         %PUT "TABLING PGM JOB SPECIFICATION WARNING:";
         %PUT "   unless EST_STAT value is ROM_S or ROM_C, only 1st variable in SU_ATTR value is used";
         %PUT "          EST_STAT value is &ic_est_stat";
         %PUT "          SU_ATTR  value is %UPCASE(&ic_est_stat)";
         %PUT "                            %UPCASE(%SCAN(&ic_su_attr,1,/)) will be used";
         %PUT "                            %UPCASE(%SCAN(&ic_su_attr,2,/)) will not be used";
      %END;
 
   %IF &ic_margins ^=     AND
       &ic_margins ^=YES  AND
       &ic_margins ^=N0   %THEN
      %DO;
         %set_response(ic_response,WARNING);
         %PUT "TABLING PGM JOB SPECIFICATION WARNING:";
         %PUT "   Any value for MARGINS which is not 'YES' is interpreted as 'NO'";
      %END;
 
   %IF &ic_print ^=      AND
       &ic_print ^= YES  AND
       &ic_print ^= N0   %THEN
      %DO;
         %set_response(ic_response,WARNING);
         %PUT "TABLING PGM JOB SPECIFICATION WARNING:";
         %PUT "   Any value for PRINT which is not 'YES' is interpreted as 'NO'";
      %END;

   %*Check specification of subpop data set;
   %IF (&ic_method=STRAT  OR
        &ic_method=DOUBLE    ) %THEN
      %DO;
         %IF &ic_sp ^= %THEN
            %DO;
               %set_response(ic_response,WARNING);
               %PUT "TABLING PGM JOB SPECIFICATION WARNING:";
               %PUT "   Specification of subpop data set not requried for STRAT, DOUBLE methods";
               %PUT "      Subpop data set not used with these methods";
            %END;
      %END;
   %IF (&ic_method = RANDOM OR
        &ic_method = CENSUS    ) %THEN
      %DO;

     
         %IF (&ic_method ^= CENSUS    ) %THEN
            %DO;
               %dset_exists(&ic_sp,ic_response,ic_dset_exists);
               %PUT "********** Test for existence of Supop = &ic_dset_exists";
               %IF &ic_dset_exists = YES %THEN
                  %DO; 

                     %chkvar(sp_spid,&ic_sp,&ic_sp_spid,REQUIRED,ic_response,ic_present);

                     %IF &ic_method = RANDOM OR
                         &ic_method = CENSUS     %THEN
                        %DO;
                           %chkvar(sp_sz,&ic_sp,&ic_sp_sz,REQUIRED,ic_response,ic_present);
                           %IF ic_present = YES %THEN 
                               %chktype(&ic_sp,&ic_sp_sz,NUMBER,ic_response);
                        %END;

                     %IF &ic_method = RANDOM %THEN
                        %DO;
                           %chkvar(sp_ssz,&ic_sp,&ic_sp_ssz,REQUIRED,ic_response,ic_present);
                           %IF ic_present = YES %THEN 
                               %chktype(&ic_sp,&ic_sp_ssz,NUMBER,ic_response);
                        %END;

                  %END;
               %ELSE 
                   %DO;
                     %set_response(ic_response,ERROR);
                     %PUT "IC_RESPONSE set to &ic_response";
                     %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
                     %PUT "   Subpop data set required";
                  %END;
            %END;

         %IF &ic_method = CENSUS OR
             &ic_method = STRAT  OR
             &ic_method = DOUBLE     %THEN
            %DO;
               %chkvar(sp_ssz,&ic_sp,&ic_sp_ssz,UNUSED,ic_response,ic_present);
            %END;

      %END;


   %*Check specification of stratum data set;
   %IF &ic_method=DOUBLE OR
       &ic_method=STRAT      %THEN
      %DO;

         %dset_exists(&ic_st,ic_response,ic_dset_exists);
         %IF &ic_dset_exists = YES %THEN
            %DO;

               %chkvar(st_spid,&ic_st,&ic_st_spid,REQUIRED,ic_response,ic_present);

               %chkvar(st_stid,&ic_st,&ic_st_stid,REQUIRED,ic_response,ic_present);

               %chkvar(st_sz,&ic_st,&ic_st_sz,REQUIRED,ic_response,ic_present);
               %IF ic_present = YES %THEN 
                   %chktype(&ic_st,&ic_st_sz,NUMBER,ic_response);

               %chkvar(st_ssz,&ic_st,&ic_st_ssz,REQUIRED,ic_response,ic_present);
               %IF ic_present = YES %THEN 
                   %chktype(&ic_st,&ic_st_ssz,NUMBER,ic_response);

	       %IF &ic_method=DOUBLE %THEN
		  %DO;

		     %chkvar(st_ssz1,&ic_st,&ic_st_ssz1,REQUIRED,ic_response,ic_present);
		     %IF ic_present = YES %THEN 
			 %chktype(&ic_st,&ic_st_ssz1,NUMBER,ic_response);

		  %END;

	       %IF &ic_method=STRAT %THEN
		  %DO;

		     %chkvar(st_ssz1,&ic_st,&ic_st_ssz1,UNUSED,ic_response,ic_present);

		  %END;

            %END;
         %ELSE
            %DO;
               %set_response(ic_response, ERROR);
               %PUT "TABLING PGM JOB SPECIFICATION ERROR:";
               %PUT "   stratification data set is required";
            %END;

      %END;

   %IF &ic_method=CENSUS OR
       &ic_method=RANDOM    %THEN
      %DO;
         %IF %symchk(ic_st)=YES %THEN
            %DO;
               %set_response(ic_response,WARNING);
               %PUT "TABLING PGM JOB SPECIFICATION WARNING:";
               %PUT "   stratum data set not needed for CENSUS, RANDOM methods";
            %END;
      %END;


   %*Check specification of sample unit evaluation data set;
   %dset_exists(&ic_su, ic_response, ic_dset_exists);
   %IF &ic_dset_exists = YES %THEN
      %DO;

         %chkvar(su_suid,&ic_su,&ic_su_suid,REQUIRED,ic_response,ic_present);

         %*Check existence and type of 1st specified analysis variable;
         %LET ic_var1 = %SCAN(&ic_su_attr,1,/);
         %chkvar(su_attr 1st var,&ic_su,&ic_var1,REQUIRED,ic_response,ic_present);
         %IF &ic_present^=YES  %THEN
             %chktype(&ic_su,&ic_var1,NUMBER,ic_response);

         %*For ROM estimates, check existence and type of specified denominator;
         %IF  &ic_su,%SCAN(&ic_su_attr,2,/) ^=       AND
             (&ic_est_stat                   = ROM_S OR
              &ic_est_stat                   = ROM_C     ) %THEN
            %DO;
               %LET ic_var2 = %SCAN(&ic_su_attr,2,/);
               %chkvar(su_attr 2d var,&ic_su,&ic_var2,REQUIRED,ic_response,ic_present);
               %IF &ic_present =YES  %THEN
                  %DO;
                     %chktype(&ic_su,&ic_var2,NUMBER,ic_response);
                  %END;
               %ELSE
                  %PUT "   the specified ROM denominator var was not found on the obs data set";

            %END;

         %IF &ic_method=CENSUS OR
             &ic_method=RANDOM    %THEN
             %chkvar(su_stid,&ic_su,&ic_su_stid,UNUSED,ic_response,ic_present);

      %END;

%PUT "IN_CHK macro: final response is %UPCASE(&ic_response)";

%LET &ic_var_out = &ic_response;

%MEND in_chk;

**END OF MACRO;
**END OF MACRO;

