/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program calculates population estimates for each cell
         in a table using a simple random sampling approach.

         An estimator is used which assumes each cell of the final
         table of estimates to represent summary statistics for ALL
         sample units in the population.  Cell estimates
         represent the population estimate of a variable which takes
         zero values for  sample units which do not satisfy
         the conditions indicated BY row and column categories in the
         table.

         At present, the program assumes two input DATA sets, one
         containing observations at the sample unit level, and one
         containing summary information about subpopulations which
         will contribute to the derived overall population estimates.

$RCSfile: random.sas,v $; 
$Revision: 1.11 $; 
$Date: 2002/02/15 21:13:46 $;   
$Author: lbednar $; 
***********************************************************************/

%MACRO random( r_est_stat =,
               r_sp       =,
               r_sp_spid  =,
               r_sp_sz    =,
               r_sp_ssz   =,
               r_su       =,
               r_su_spid  =,
               r_su_suid  =,
               r_su_rows  =,
               r_su_cols  =,
               r_su_attr  =,
               r_unit     =,
               r_margins  =  );

   /***********************************************************************/
   /* This macro uses the following macro variables which should          */
   /* already be defined in the macro referencing environment which       */
   /* invokes this macro:                                                 */
   /***********************************************************************/

   %PUT Macro RANDOM started;

   %LET r_est_stat = %UPCASE(&r_est_stat);
   %LET r_margins  = %UPCASE(&r_margins);

   %*Retrieve input DATA into working DATA sets;
      %get_sp_r(gsr_data_in  =&r_sp,
                gsr_id       =&r_sp_spid,
                gsr_sz       =&r_sp_sz,
                gsr_ssz      =&r_sp_ssz,
                gsr_data_out =WORK.subpop);

      %get_ob(go_data_in  =&r_su,
              go_subpop   =&r_su_spid,
              go_id       =&r_su_suid,
              go_r_vars   =&r_su_rows,
              go_c_vars   =&r_su_cols,
              go_an       =&r_su_attr,
              go_an_denom =,
              go_unit     =&r_unit,
              go_data_out =WORK.obs);
   
   *Delete data sets to be used;
   PROC DATASETS LIBRARY=work;
       DELETE m_su
              m_su_m
              m_su_r
              m_su1
              m_su2
              m_sp;
   RUN;

   OPTIONS MPRINT SYMBOLGEN;
   
   *************************** STEP 1 ****************************;
   *STEP 1 - Calculate relative subpopulation weights in combined
     estimate;
   
   *Calculate relative weights of subpopulations;
   PROC SUMMARY DATA=work.subpop;
      VAR sz_sp;
      CLASS sp;
      OUTPUT OUT=work.m_sp
             SUM=sz_sp;
   RUN;
   DATA work.m_sp(LABEL="Subpop attributes");
      SET work.m_sp;
      RETAIN sz_pop 0;
      IF _type_=0 THEN sz_pop=sz_sp;
      ELSE IF _type_=1 THEN
         DO;
            wt_sp=sz_sp/sz_pop;
            OUTPUT;
         END;
      DROP _type_ _freq_;
   RUN;
   
   PROC SORT DATA=work.subpop;
      BY sp;
   RUN;
   PROC SORT DATA=work.m_sp;
      BY sp;
   RUN;
   DATA work.m_sp;
      MERGE work.m_sp
            work.subpop;
      f=n/sz_sp;
      LABEL sp    ="Subpop ID"
            sz_pop="Pop size"
            sz_sp ="Subpop size"
            wt_sp ="Subpop wt"
            f     ="Subpop samp fraction"
            n     ="Subpop samp size";
   RUN;
   TITLE3 "Subpopulation attributes";
   PROC PRINT DATA=work.m_sp;
      VAR sp sz_pop sz_sp wt_sp f n;
   RUN;
   
   ************************ STEP 2 ****************************;
   *STEP 2 - Calculate subpopulation sample summary statistics;
   
   *Make sure that each sample unit has an observation for each
    possible combination of row and column in the input observation data set;
   
   *Generate a data set containing all response combinations observed
    in observation data set;
   PROC SORT DATA=work.obs;
      BY &r_su_cols 
         &r_su_rows;
   RUN;
   PROC SUMMARY DATA=work.obs;
      VAR attrib;
      BY &r_su_cols 
         &r_su_rows;
      OUTPUT OUT=work.m_su_r (LABEL="Response categories"
                              DROP =_FREQ_ _TYPE_ n)
             N = n;
   RUN;
   
   *Sum any subcategorizations of sample unit observations within
    input observations to level of specified response category variables;
   PROC SORT DATA=work.obs;
      BY sp su_id 
         &r_su_cols 
         &r_su_rows;
   RUN;
   PROC SUMMARY DATA=work.obs;
      VAR attrib;
      %IF %UPCASE(&r_margins)=YES %THEN
         %DO;
            *These statments result in generation of estimates for
             marginal populations;
            BY sp 
               su_id;
            CLASS &r_su_cols 
                  &r_su_rows;
         %END;
      %ELSE
         %DO;
            *This statement results in generation of estimates for
             only those response categories provided in the input
             observation data set;
            BY sp 
               su_id 
               &r_su_cols 
               &r_su_rows;
         %END;
      OUTPUT OUT=work.m_su (DROP=_TYPE_ _FREQ_)
             SUM=attrib;
   RUN;
   
   
   *Join the two data sets just created to form a data set where there
    is an observation for each possible response category within each
    sample unit;
   PROC SORT DATA=work.m_su;
      BY &r_su_cols 
         &r_su_rows;
   RUN;
   DATA work.m_su_m (LABEL="Merged su attributes/categories");
      MERGE work.m_su
            work.m_su_r;
      BY &r_su_cols 
         &r_su_rows;
      IF attrib<=.z THEN attrib=0;
   RUN;
   
   *Change all blank row and col var values to 'ALL';
   %chg_blnk(data_in  =work.m_su_m,
             chg_vars =&r_su_rows &r_su_cols,
             new_val  =ALL,
             data_out =work.m_su_m)
   
   *Subpopulation-level sums for each combination of row and column response
    categories;
   PROC SORT DATA=work.m_su_m;
      BY sp 
         &r_su_cols 
         &r_su_rows 
         su_id;
   RUN;
   PROC SUMMARY DATA=work.m_su_m;
      VAR    attrib;
      BY     sp 
             &r_su_cols 
             &r_su_rows;
      OUTPUT OUT=m_su1 (LABEL="Subpop samp sums"
                        DROP =_TYPE_             )
             USS=uss
             SUM=sum;
   RUN;
   
   
   
   DATA work.m_su1(LABEL="Subpop samp summary stats");
      MERGE work.m_su1
            work.m_sp;
      BY sp;
      avg=sum/n;
      IF n>1
         THEN s2 = (uss - n*avg**2) / (n-1);
      ELSE IF n=1
         THEN s2 = 0;
      ELSE    s2 =.c;
      var = (1-f) * s2/n;
      %IF &r_est_stat=MEAN %THEN
         %DO;
             est_sp = avg;
             var_sp = var;
         %END;
      %ELSE %IF &r_est_stat=SUM %THEN
         %DO;
             est_sp = sz_sp    * avg;
             var_sp = sz_sp**2 * var;
         %END;
      LABEL sum   ="Subpop samp sum"
            avg   ="Subpop samp mean"
            s2    ="Subpop sample var"
            est_sp="Subpop &r_su_attr &r_est_stat est"
            var_sp="Estd var of subpop &r_su_attr &r_est_stat est"
            uss   ="Subpop uncorr SS";
   RUN;
   
   ***************************** STEP 3 ******************************;
   *STEP 3 - Calculate weighted subpop contributions to pop estimates;
   
   DATA m_su2(LABEL="Subpop contribs to pop est");
       SET m_su1;
       %IF &r_est_stat=MEAN %THEN
           %DO;
               w_est_sp = wt_sp    * est_sp;
               w_var_sp = wt_sp**2 * var_sp;
           %END;
       %IF &r_est_stat=SUM %THEN
           %DO;
               w_est_sp = est_sp;
               w_var_sp = var_sp;
           %END;
       LABEL w_est_sp="Wtd subp contrib to &r_su_attr &r_est_stat est"
             w_var_sp="Wtd subp contrib to estd var of pop est";
   RUN;
   PROC SORT DATA=m_su2;
       BY &r_su_cols 
          &r_su_rows 
          sp;
   RUN;
   TITLE3 "Subpopulation sample summary statistics";
   PROC PRINT DATA=m_su2
              UNIFORM;
       VAR sp n est_sp var_sp wt_sp w_est_sp w_var_sp;
       SUM wt_sp w_est_sp w_var_sp;
       BY &r_su_cols 
          &r_su_rows;
   RUN;
   
   **************************** STEP 4 ******************************;
   *STEP 4 - Calculate sum of subpop contribs to obtain pop
    parameter estimates;
   
   PROC SORT DATA=work.m_su2;
       BY &r_su_rows 
          &r_su_cols;
   RUN;
   PROC SUMMARY DATA=work.m_su2;
       VAR w_est_sp w_var_sp;
       BY &r_su_rows 
          &r_su_cols;
       OUTPUT OUT=m_final(LABEL="Pop estimates"
                          DROP=_type_ _freq_)
              SUM=&r_est_stat
                  v_&r_est_stat;
   RUN;
   DATA work.m_final(LABEL="Pop estimates");
      SET work.m_final;
      sd_&r_est_stat=SQRT(v_&r_est_stat);
      LABEL &r_est_stat="Pop &r_su_attr &r_est_stat est"
            v_&r_est_stat="Estd var of pop &r_su_attr &r_est_stat est";
   RUN;
   
   PROC DATASETS LIBRARY=work;
       *DELETE m_su
               m_su1
               m_su2
               m_sp;
       *Data SET M_FINAL is preserved for passing DATA to REPOUT macro;
   RUN;

   %PUT Macro RANDOM ended;

%MEND random;

%*The following macro invocation is used for development testing;
%*random(r_est_stat =,
        r_sp       =,
        r_sp_spid  =,
        r_sp_sz    =,
        r_sp_ssz   =,
        r_su       =,
        r_su_spid  =,
        r_su_suid  =,
        r_su_rows  =,
        r_su_cols  =,
        r_su_attr  =,
        r_unit     =,
        r_margins  =   );

*** END OF MACRO ***;
*** END OF MACRO ***;
