/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program calculates population estimates for each cell
         in a table using a simple random sampling approach.

         An estimator is used which assumes each cell of the final
         table of estimates to represent summary statistics for ALL
         sample units in the population.  Cell estimates
         represent the population estimate of a variable which takes
         zero values for  sample units which do not satisfy
         the conditions indicated BY row and column categories in the
         table.

         At present, the program assumes two input DATA sets, one
         containing observations at the sample unit level, and one
         containing summary information about subpopulations which
         will contribute to the derived overall population estimates.

$RCSfile: random_rs.sas,v $; 
$Revision: 1.10 $; 
$Date: 2002/02/15 21:15:11 $;   
$Author: lbednar $; 
***********************************************************************/

%MACRO random_rs( rrs_est_stat      =,
                  rrs_sp            =,
                  rrs_sp_spid       =,
                  rrs_sp_sz         =,
                  rrs_sp_ssz        =,
                  rrs_su            =,
                  rrs_su_spid       =,
                  rrs_su_suid       =,
                  rrs_su_rows       =,
                  rrs_su_cols       =,
                  rrs_su_attr       =,
                  rrs_su_attr_denom =,
                  rrs_unit          =,
                  rrs_margins       =  );

   /***********************************************************************/
   /* This macro uses the following macro variables which should          */
   /* already be defined in the macro referencing environment which       */
   /* invokes this macro:                                                 */
   /***********************************************************************/

   %PUT Macro RANDOM_RS started;

   %LET rrs_est_stat = %UPCASE(&rrs_est_stat);
   %LET rrs_margins  = %UPCASE(&rrs_margins);

   %*Retrieve input DATA into working DATA sets;
   %get_sp_r(gsr_data_in  =&rrs_sp,
             gsr_id       =&rrs_sp_spid,
             gsr_sz       =&rrs_sp_sz,
             gsr_ssz      =&rrs_sp_ssz,
             gsr_data_out =WORK.subpop);

   %get_ob(go_data_in  =&rrs_su,
           go_subpop   =&rrs_su_spid,
           go_id       =&rrs_su_suid,
           go_r_vars   =&rrs_su_rows,
           go_c_vars   =&rrs_su_cols,
           go_an       =&rrs_su_attr,
           go_an_denom =&rrs_su_attr_denom,
           go_unit     =&rrs_unit,
           go_data_out =WORK.obs);
   
   *Delete data sets to be used;
   PROC DATASETS LIBRARY=WORK;
       DELETE m_su
              m_su_m
              m_su1
              m_su2
              m_sp;
   RUN;

   OPTIONS MPRINT SYMBOLGEN;
   
   *************************** STEP 1 ****************************;
   *STEP 1 - Calculate relative subpopulation weights in combined
     estimate;
   
   *Calculate relative weights of subpopulations;
   PROC SUMMARY DATA=WORK.subpop;
      VAR sz_sp;
      CLASS sp;
      OUTPUT OUT = WORK.m_sp
             SUM = sz_sp;
   RUN;
   DATA WORK.m_sp(LABEL="Subpop attributes");
      SET WORK.m_sp;
      RETAIN sz_pop 0;
      IF _type_=0 THEN sz_pop=sz_sp;
      ELSE IF _type_=1 THEN
         DO;
            wt_sp=sz_sp/sz_pop;
            OUTPUT;
         END;
      DROP _type_ _freq_;
   RUN;
   
   PROC SORT DATA=WORK.subpop;
      BY sp;
   RUN;
   PROC SORT DATA=WORK.m_sp;
      BY sp;
   RUN;
   DATA WORK.m_sp;
      MERGE WORK.m_sp
            WORK.subpop;
      f=n/sz_sp;
      LABEL sp    ="Subpop ID"
            sz_pop="Pop size"
            sz_sp ="Subpop size"
            wt_sp ="Subpop wt"
            f     ="Subpop samp fraction"
            n     ="Subpop samp size";
   RUN;
   TITLE3 "Subpopulation attributes";
   PROC PRINT DATA=WORK.m_sp;
      VAR sp sz_pop sz_sp wt_sp f n;
   RUN;
   
   ************************ STEP 2 ****************************;
   *STEP 2 - Calculate subpopulation sample summary statistics;

   *Sum any subcategorizations of sample unit observations within
    input observations to level of specified response category variables;
   PROC SORT DATA=WORK.obs;
      BY sp 
         su_id 
         &rrs_su_rows 
         &rrs_su_cols;
   RUN;
   PROC SUMMARY DATA=WORK.obs;
      VAR attrib
          attrib2;
      %IF %UPCASE(&rrs_margins)=YES %THEN
         %DO;
            *These statments result in generation of estimates for
             marginal populations;
            BY sp 
               su_id;
            CLASS &rrs_su_cols 
                  &rrs_su_rows;
         %END;
      %ELSE
         %DO;
            *This statement results in generation of estimates for
             only those response categories provided in the input
             observation data set;
            BY sp 
               su_id 
               &rrs_su_rows 
               &rrs_su_cols;
         %END;
      OUTPUT OUT = WORK.m_su (DROP = _TYPE_ _FREQ_)
             SUM = y
                   x;
   RUN;
   DATA WORK.m_su_m;
      SET WORK.m_su;
      yx = y * x;
   RUN;
   
   *Change all blank row and col var values to 'ALL';
   %chg_blnk(data_in  =WORK.m_su_m,
             chg_vars =&rrs_su_rows &rrs_su_cols,
             new_val  =ALL,
             data_out =WORK.m_su_m)
   
   *Subpopulation-level sums for each combination of row and column response
    categories;
   PROC SORT DATA=WORK.m_su_m;
      BY sp 
         &rrs_su_rows 
         &rrs_su_cols 
         su_id;
   RUN;
   PROC SUMMARY DATA=WORK.m_su_m;
      VAR    y
             x
             yx;
      BY     sp 
             &rrs_su_rows 
             &rrs_su_cols;
      OUTPUT OUT = WORK.m_su1 (LABEL="Subpop samp sums"
                               DROP =_TYPE_ _FREQ_ yx_uss )
             USS = y_uss
                  x_uss
                  yx_uss
             SUM =y_sum
                  x_sum
                  yx_sum;
   RUN;
   
   DATA WORK.m_su1(LABEL="Subpop samp summary stats");
      MERGE WORK.m_su1
            WORK.m_sp;
      BY sp;
      y_avg = y_sum / n;
      x_avg = x_sum / n;
      IF n>1 THEN 
         DO;
            y_s2 = (y_uss - n*y_avg**2) / (n-1);
            x_s2 = (x_uss - n*x_avg**2) / (n-1);
         END;
      ELSE IF n=1 THEN
         DO;
            y_s2 = 0;
            x_s2 = 0;
         END;
      ELSE
         DO;
            y_s2 =.c;
            x_s2 =.c;
         END;
      r      = y_sum / x_sum;
      cov_yx = (yx_sum - n*x_avg*y_avg) / (n-1);
      var_sp = ((1-f)/ (n*x_avg**2))*(y_s2+(r**2)*(x_s2)-2*r*cov_yx);
      est_sp = r; 
      LABEL n       = "Suppop samp size"
            x_sum   = "Subpop samp x mean"
            y_sum   = "Subpop samp y mean"
            x_avg   = "Subpop samp x mean"
            y_avg   = "Subpop samp y mean"
            x_s2    = "Subpop sample x var"
            y_s2    = "Subpop sample y var"
            r       = "Subpop &rrs_su_attr./&rrs_su_attr_denom &rrs_est_stat est"
            est_sp  = "Subpop &rrs_su_attr./&rrs_su_attr_denom &rrs_est_stat est"
            var_sp  = "Estd var of subpop &rrs_su_attr./&rrs_su_attr &rrs_est_stat est"
            x_uss   = "Subpop uncorr x SS"
            y_uss   = "Subpop uncorr y SS"
            cov_yx  = "Covariance of y with x";
   RUN;

   ***************************** STEP 3 ******************************;
   *STEP 3 - Calculate weighted subpop contributions to pop estimates;
   
   DATA WORK.m_su2 ( LABEL = "Subpop contribs to pop est" );
       SET WORK.m_su1;
       w_est_sp = wt_sp    * est_sp;
       w_var_sp = wt_sp**2 * var_sp;
       LABEL w_est_sp = "Wtd subp contrib to &rrs_su_attr/&rrs_su_attr_denom &rrs_est_stat est"
             w_var_sp = "Wtd subp contrib to estd var of pop est";
   RUN;
   PROC SORT DATA=WORK.m_su2;
       BY sp
          &rrs_su_rows 
          &rrs_su_cols;
   RUN;
   TITLE3 "Subpopulation sample summary statistics";
   PROC PRINT DATA=WORK.m_su2
              UNIFORM;
       VAR sp &rrs_su_rows &rrs_su_cols n est_sp var_sp wt_sp w_est_sp w_var_sp;
       BY  sp;
   RUN;
   
   PROC PRINT DATA=WORK.m_su2
              UNIFORM;
       VAR sp &rrs_su_rows &rrs_su_cols n  y_sum  x_sum   y_uss  x_uss 
                                           y_avg  x_avg   y_s2   x_s2   
                                           yx_sum cov_yx;
       BY  sp;
   RUN;
   
   **************************** STEP 4 ******************************;
   *STEP 4 - Calculate sum of subpop contribs to obtain pop
    parameter estimates;
   
   PROC SORT DATA=WORK.m_su2;
       BY &rrs_su_rows 
          &rrs_su_cols;
   RUN;
   PROC SUMMARY DATA=WORK.m_su2;
       VAR w_est_sp 
           w_var_sp;
       BY &rrs_su_rows 
          &rrs_su_cols;
       OUTPUT OUT = WORK.m_final ( LABEL="Pop estimates"
                                   DROP =_TYPE_ _FREQ_  )
              SUM = &rrs_est_stat
                    v_&rrs_est_stat;
   RUN;
   DATA WORK.m_final ( LABEL = "Pop estimates" );
      SET WORK.m_final;
      sd_&rrs_est_stat=SQRT(v_&rrs_est_stat);
      LABEL &rrs_est_stat   = "Pop &rrs_su_attr./&rrs_su_attr_denom &rrs_est_stat est"
            v_&rrs_est_stat = "Estd var of pop &rrs_su_attr &rrs_est_stat est";
   RUN;
   
   PROC DATASETS LIBRARY=WORK;
       DELETE m_su
              m_su_m
              m_su1
              m_su2
              m_sp;
       *Data SET M_FINAL is preserved for passing DATA to REPOUT macro;
   RUN;

   %PUT Macro RANDOM_RS ended;

%MEND random_rs;

%*The following macro invocation is used for development testing;
%*random_rs(rrs_est_stat =,
          rrs_sp       =,
          rrs_sp_spid  =,
          rrs_sp_sz    =,
          rrs_sp_ssz   =,
          rrs_su       =,
          rrs_su_spid  =,
          rrs_su_suid  =,
          rrs_su_rows  =,
          rrs_su_cols  =,
          rrs_su_attr  =,
          rrs_unit     =,
          rrs_margins  =   );

*** END OF MACRO ***;
*** END OF MACRO ***;
