/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program defines a SAS macro program which produces a
         table report of population estimates derived from PRIME Program
         inventory data, using the SAS REPORT procedure.  Row and column
         totals presented in the output report tables are estimated
         population parameters for marginal populations.

         At present, the program assumes one input DATA set, which
         should be created in the estimation module of the PRIME Table
         Reporting Application.

$RCSfile: repout.sas,v $; 
$Revision: 1.6 $; 
$Date: 2002/02/20 01:46:01 $;   
$Author: lbednar $; 
******************************* CODE **********************************/

%MACRO repout(r2_method        =,
              r2_est_stat      =,
              r2_rows          =,
              r2_cols          =,
              r2_su_attr       =,
              r2_su_attr_denom =,
              r2_unit          =,
              r2_sig           =3,
              r2_htmlout       =);

/***********************************************************************/
/* This macro uses the following macro variables, which should already */
/* be defined in the macro referencing environment which invokes the   */
/* macro:                                                              */
/*  METHOD   - used to set appropriate title statement text            */
/*  SU_ATTR  - contains the name of the analysis variable              */
/*  UNIT     - indicates units used in report (e.g. 100s, 1000s,...)   */
/*  EST_STAT - indicates the population statistic estimated            */
/*  SIG      - number of significant digits in final figures           */
/*  HTMLOUT  - base name for files to hold HTML versions of output     */
/***********************************************************************/

%PUT Macro REPOUT started;

%LOCAL r2_col1
       r2_col2
       r2_row1
       r2_row2
       r2_row3
       r2_rtext
       r2_attr_desc;

%IF &r2_est_stat =ROM_S %THEN %LET r2_attr_desc = &r2_su_attr/&r2_su_attr_denom;
%ELSE                         %LET r2_attr_desc = &r2_su_attr;

%*Extract individual row and column variable names from the multiple
  specifications in macro variables COLS and ROWS;
   %LET r2_col1 = %SCAN(&r2_cols, 1);
   %LET r2_col2 = %SCAN(&r2_cols, 2);
   %LET r2_row1 = %SCAN(&r2_rows, 1);
   %LET r2_row2 = %SCAN(&r2_rows, 2);
   %LET r2_row3 = %SCAN(&r2_rows, 3);
   %LET r2_su_attr = %UPCASE(&r2_su_attr);

*Delete any preexisting versions of the data sets used in this macro;
PROC DATASETS LIBRARY = WORK;
    DELETE m_cvals 
           m_ncols;
    DELETE m_repdat 
           m_cvals 
           m_ncols 
           m_final2;
RUN;

%*Remove observations from input data set which represent unwanted totals
  over first column variable;
DATA WORK.m_final2;
   SET WORK.m_final;
   %IF &r2_row2 ^=  %THEN
      %DO;
         WHERE NOT ( &r2_row1 = 'ALL' AND &r2_row2^ = 'ALL' );
      %END;
RUN;

%*Calculate number of columns to be used in report, place value
  in macro variable NCOLS;
%IF &r2_cols ^= %THEN
   %DO;
      PROC SORT DATA = WORK.m_final2;
         BY &r2_cols;
      RUN;
      PROC SUMMARY DATA = WORK.m_final2;
         VAR &r2_est_stat;
         BY &r2_cols;
         OUTPUT OUT = WORK.m_cvals n = n;
      RUN;
      PROC SUMMARY DATA = WORK.m_cvals;
         VAR n;
         OUTPUT OUT = WORK.m_ncols n = n;
      RUN;
      DATA WORK.m_ncols;
         SET WORK.m_ncols;
         CALL SYMPUT( 'ncols', n);
      RUN;
   %END;
%ELSE %LET r2_ncols = 1;

%IF       &r2_method = DOUBLE
   %THEN %LET r2_rtext = double sample;
%ELSE %IF &r2_method = STRAT
   %THEN %LET r2_rtext = stratified sample;
%ELSE %IF &r2_method = RANDOM
   %THEN %LET r2_rtext = simple random sample;
%ELSE %IF &r2_method = CENSUS
   %THEN %LET r2_rtext = complete census;

******************** TABLE OF POP PARAMETER ESTIMATE ******************;
*This REPORT specification is designed to provide estimates of marginal
 populations rather than simple row and column sums;

TITLE3 "Estimated population &r2_est_stat of variable &r2_attr_desc, in &r2_unit.s";
TITLE4 "Estimated to &r2_sig significant digits from a &r2_rtext";

%IF &r2_htmlout ^= %THEN
   %DO;
      ODS TRACE OUTPUT;
      ODS HTML BODY  = "&r2_htmlout._est.html"
               STYLE = MINIMAL;
   %END;

PROC REPORT DATA = WORK.m_final2
            NOWINDOWS
            MISSING;

   RBREAK BEFORE / SKIP OL;

   %*Define row groups and determine 1st col summed for row total;
   %IF &r2_row1 ^= %THEN
      %DO;
         DEFINE &r2_row1 / GROUP;
      %END;
   %IF &r2_row2 ^= %THEN
      %DO;
        DEFINE &r2_row2 / GROUP;
        BREAK AFTER &r2_row1 / SKIP;
      %END;
   %IF &r2_row3^= %THEN
      %DO;
         DEFINE &r2_row3 / GROUP;
      %END;

   %*Define column groups and associated statements;
   %IF &r2_col2^= %THEN
      %DO;
         COLUMN &r2_row1 &r2_row2 &r2_row3 &r2_est_stat,(&r2_col1, &r2_col2);
         DEFINE &r2_col1/ACROSS;
         DEFINE &r2_col2/ACROSS;
      %END;
   %ELSE %IF &r2_col1^= %THEN
      %DO;
         COLUMN &r2_row1 &r2_row2 &r2_row3 &r2_est_stat, &r2_col1;
         DEFINE &r2_col1/ACROSS;
      %END;
   %ELSE
      %DO;
         COLUMN &r2_row1 &r2_row2 &r2_row3 &r2_est_stat;
      %END;

   DEFINE &r2_est_stat / ANALYSIS FORMAT = COMMA15. WIDTH = 17;

RUN;

%IF &r2_htmlout ^=  %THEN
   %DO;
      ODS HTML CLOSE;
      ODS TRACE OFF;
   %END;

********************* TABLE OF ESTIMATED STD DEV ****************************;
*This REPORT specification is designed to provide std error of marginal
 population estimates provided above;

%IF &r2_htmlout ^= %THEN
   %DO;
      ODS TRACE OUTPUT;
      ODS HTML BODY  = "&r2_htmlout._std.html"
               STYLE = MINIMAL;
   %END;

TITLE3 "Estimated standard deviation of estimated &r2_attr_desc &r2_est_stat, in &r2_unit.s";
TITLE4 "Estimated to &r2_sig significant digits from a &r2_rtext";

PROC REPORT DATA = WORK.m_final2
            NOWINDOWS
            MISSING;

   RBREAK BEFORE / SKIP OL;
   *rbreak after / ol summarize;

   %*Define row groups and determine 1st col summed for row total;
   %IF &r2_row1 ^= %THEN
      %DO;
         DEFINE &r2_row1/GROUP;
      %END;
   %IF &r2_row2 ^= %THEN
      %DO;
         DEFINE &r2_row2 / GROUP;
         BREAK AFTER &r2_row1 / SKIP;
      %END;
   %IF &r2_row3 ^=  %THEN
      %DO;
         DEFINE &r2_row3 / GROUP;
      %END;

   %*Define column groups and associated statements;
   %IF &r2_col2 ^=  %THEN
      %DO;
         COLUMN &r2_row1 &r2_row2 &r2_row3 sd_&r2_est_stat, (&r2_col1, &r2_col2);
         DEFINE &r2_col1 / ACROSS;
         DEFINE &r2_col2 / ACROSS;
      %END;
   %ELSE %IF &r2_col1^= %THEN
      %DO;
         COLUMN &r2_row1 &r2_row2 &r2_row3 sd_&r2_est_stat, &r2_col1;
         DEFINE &r2_col1 / ACROSS;
      %END;
   %ELSE
      %DO;
         COLUMN &r2_row1 &r2_row2 &r2_row3 sd_&r2_est_stat;
      %END;

   DEFINE sd_&r2_est_stat / ANALYSIS FORMAT = COMMA15. WIDTH = 17;

RUN;

%IF &r2_htmlout ^=  %THEN
   %DO;
      ODS HTML CLOSE;
      ODS TRACE OFF;
   %END;

*Delete data sets created by this macro;
PROC DATASETS LIBRARY = WORK;
    DELETE m_cvals  
           m_ncols;
    DELETE m_repdat 
           m_cvals 
           m_ncols 
           m_final2;
RUN;

%PUT Macro REPOUT ended;

%MEND repout;

%*repout(r2_method        =,
	r2_est_stat      =,
	r2_rows          =,
	r2_cols          =,
	r2_su_attr       =,
	r2_su_attr_denom =,
	r2_unit          =,
        r2_sig           =,
	r2_htmlout       =);

*** END OF MACRO ***;
*** END OF MACRO ***;
