/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program calculates population estimates for each cell
         in a table using a simple stratified sampling approach.

         An estimator is used which assumes each cell of the final
         table of estimates to represent summary statistics for ALL
         secondary sample units in the population.  Cell estimates
         represent the population estimate of a variable which takes
         zero values for secondary sample units which do not satisfy
         the conditions indicated by row and column categories in the
         table.

         At present, the program assumes three input data sets, one
         containing observations at the sample unit level, one
         containing summary statistics for strata, and one containing
         summary information about subpopulations which will contribute
         to the derived overall population estimates.

$RCSfile: strat.sas,v $; 
$Revision: 1.10 $; 
$Date: 2002/02/15 21:16:44 $;   
$Author: lbednar $; 
***********************************************************************/

%macro strat(s_est_stat =,
	     s_st       =,
	     s_st_spid  =,
	     s_st_stid  =,
	     s_st_ssz   =,
	     s_st_sz    =,
	     s_su       =,
	     s_su_suid  =,
	     s_su_stid  =,
	     s_su_rows  =,
	     s_su_cols  =,
	     s_su_attr  =,
	     s_unit     =,
	     s_margins  =);

%*For use in testing/development which excludes use of TABLING macro -
  to be REMOVED otherwise;
%*Separate 1st and 2d variables specified in value of T_SU_ATTR - for 
  use in ratio estimators;
%LET s_su_attr_denom = %SCAN(&s_su_attr,2,/);
%LET s_su_attr       = %SCAN(&s_su_attr,1,/);

%put Macro STRAT started;

%*Change to uppercase to facilitate comparisons;
%LET s_est_stat = %UPCASE(&s_est_stat);
%LET s_margins  = %UPCASE(&s_margins);

%*Retrieve input data into working DATA sets;
   %*Revision 1.6 eliminates the need for running GET_SP;
   %get_st_s(gss_data_in  = &s_st,
             gss_sp       = &s_st_spid,
             gss_id       = &s_st_stid,
             gss_sz       = &s_st_sz,
             gss_ssz      = &s_st_ssz,
             gss_data_out = WORK.STRATUM);
   %get_ob_s(gos_data_in  = &s_su,
             gos_st       = &s_su_stid,
             gos_id       = &s_su_suid,
             gos_r_vars   = &s_su_rows,
             gos_c_vars   = &s_su_cols,
             gos_an       = &s_su_attr,
             gos_an_denom =,
             gos_unit     = &s_unit,
             gos_data_out = WORK.OBS);

*The program assumes existence of input DATA sets WORK.SUBPOP, WORK.STRATA
 and WORK.OBS;

PROC DATASETS LIBRARY = work;
    delete m_su1
           m_su2
           m_su3
           m_su4
           m_sp
           m_st
           m_temp
           m_final;
RUN;

*Calculate weights of subpopulations relative to complete target
 population;
PROC SUMMARY DATA = WORK.stratum;
   VAR    sz_h;
   CLASS  sp;
   OUTPUT OUT = WORK.m_sp
          SUM = sz_sp;
RUN;
DATA WORK.m_sp (LABEL = "Subpop attributes");
   SET    WORK.m_sp;
   RETAIN sz_pop 0;
   IF _TYPE_ = 0
      THEN sz_pop = sz_sp;
   ELSE IF _TYPE_ = 1 THEN
      DO;
         wt_sp = sz_sp / sz_pop;
         OUTPUT;
      END;
   DROP _TYPE_ _FREQ_;
RUN;

*Calculate subpop sample sizes;
PROC SORT DATA = WORK.stratum;
    BY sp h;
RUN;
PROC SUMMARY DATA = WORK.stratum;
    VAR    n_h;
    BY     sp;
    OUTPUT OUT = WORK.m_temp (DROP = _TYPE_ _FREQ_)
           SUM = n;
RUN;
DATA WORK.m_sp;
   MERGE WORK.m_temp
         WORK.m_sp;
   BY    sp;
   LABEL sp     ="Subpop ID"
         sz_pop ="Pop size"
         sz_sp  ="Subpop size"
         wt_sp  ="Subpop wt"
         n      ="Subpop samp size";
RUN;
TITLE3 "Subpopulation statistics";
PROC PRINT DATA = WORK.m_sp;
   VAR sp sz_pop sz_sp wt_sp n;
RUN;

*Calculate subpopulation stratum wts;
DATA WORK.m_st(LABEL = "Stratum attributes");
   MERGE WORK.m_sp
         WORK.stratum;
   BY    sp;
   w_h = sz_h / sz_sp;
   f_h = n_h  / sz_h;
   IF h > .Z THEN OUTPUT;
   LABEL w_h = "Subpop stratum wt"
         f_h = "Subpop stratum samp frac";
   KEEP  sp h sz_h sz_sp w_h n_h f_h;
RUN;
TITLE3 "Subpop stratum statistics";
PROC PRINT DATA = WORK.m_st
           uniform;
   VAR sp h sz_h sz_sp w_h n_h f_h;
   SUM                 w_h n_h;
RUN;

*Calculate secondary sample summary statistics for strata;
*Match observations to subpopulations;
PROC SORT DATA = WORK.m_st;
   BY h;
RUN;
PROC SORT DATA = WORK.obs;
   BY h;
RUN;
DATA WORK.m_su(LABEL = "Merged stratum/su attributes");
   MERGE WORK.obs  (KEEP = h su_id &s_su_rows &s_su_cols attrib
                    IN   = obs_in)
         WORK.m_st (KEEP = sp h);
   BY    h;
   IF obs_in = 1 THEN OUTPUT;
RUN;

******************************* STEP 1 ************************************;
*STEP 1 - Calculate stratum sample sum and uncorr SS;
*Within-sample-unit sums of response categories;
PROC SORT DATA = WORK.m_su;
   BY sp 
      h 
      su_id;
RUN;
PROC SUMMARY data = WORK.m_su;
   VAR   attrib;
   BY    sp 
         h 
         su_id;
   CLASS &s_su_cols 
         &s_su_rows;
   OUTPUT OUT = WORK.m_su (DROP = _TYPE_ _FREQ_)
          SUM = attrib;
RUN;

*Statum-level sums;
PROC SORT DATA = WORK.m_su;
   BY sp 
      h 
      &s_su_cols 
      &s_su_rows;
RUN;
PROC SUMMARY DATA = WORK.m_su;
   VAR    attrib;
   BY     sp 
          h 
          &s_su_cols 
          &s_su_rows;
   OUTPUT OUT = WORK.m_su1
          USS = uss_h
          SUM = sum_h;
RUN;
DATA WORK.m_su1 (LABEL = "Stratum samp sums");
   SET   WORK.m_su1;
   LABEL uss_h = "Uncorr stratum SS"
         sum_h = "Analysis var stratum sum";
RUN;

*Change blank values of row and col variables to "ALL";
%chg_blnk(data_in  = WORK.m_su1,
          chg_vars = &s_su_rows &s_su_cols,
          new_val  = ALL,
          data_out = WORK.m_su1)

******************************* STEP 2 *********************************;
*STEP 2 - Calculate stratum contributions to subpop estimates;
PROC SORT DATA = WORK.m_su1;
    BY sp 
       h;
RUN;
PROC SORT DATA = WORK.m_st;
    BY sp 
       h;
RUN;
DATA WORK.m_su2 (LABEL = "Stratum samp mean/var");
    MERGE WORK.m_st
          WORK.m_su1;
    BY    sp 
          h;
    avg_h = sum_h / n_h;
    IF n_h > 1
       THEN s2_h = (uss_h - n_h*(avg_h)**2 )/(n_h-1);
    ELSE IF n_h = 1
       THEN s2_h = 0;
    ELSE s2_h = .c;
    var_h = ( 1 - f_h ) * s2_h / n_h;
    w_avg_h = w_h    * avg_h;
    w_var_h = w_h**2 * var_h;
    LABEL sum_h   = "Cell stratum SS"
          uss_h   = "Cell stratum uncorr SS"
          avg_h   = "Cell stratum mean"
          var_h   = "Estd var of stratum mean"
          s2_h    = "Cell stratum sample var"
          w_avg_h = "Cell wtd stratum mean"
          w_var_h = "Wtd var of stratum mean est";
    DROP _TYPE_ _FREQ_;
RUN;
PROC SORT DATA = WORK.m_su2;
    BY sp
       &s_su_rows 
       &s_su_cols
       h;
RUN;
TITLE3 "Stratum sample mean and sample variance";
TITLE4 "STRAT macro step 2";
PROC PRINT DATA = WORK.m_su2;
    VAR sp &s_su_rows &s_su_cols h avg_h n_h uss_h sum_h s2_h var_h w_h w_avg_h w_var_h;
    SUM                                  n_h                        w_h w_avg_h w_var_h;
    BY  sp;
RUN;

******************************** STEP 3 ***********************************;
*STEP 3 - Calculate subpop estimates by adding wtd stratum contributions;
PROC SORT DATA = WORK.m_su2;
    BY sp 
       &s_su_rows 
       &s_su_cols;
RUN;
PROC SUMMARY DATA = WORK.m_su2;
   VAR w_avg_h 
       w_var_h;
   BY  sp 
       &s_su_rows 
       &s_su_cols;
   OUTPUT OUT = WORK.m_su3 (LABEL = "Subpop estd mean"
                       DROP = _TYPE_ _FREQ_)
          SUM = avg_sp 
                var_sp;
RUN;

*Add subpopulation values to cell/stratum info for later use;
DATA WORK.m_su3 (LABEL = "Subpop parameter est");
   MERGE WORK.m_sp  (KEEP = sp sz_sp)
         WORK.m_su3;
   BY    sp;
   %IF &s_est_stat = MEAN %THEN
      %DO;
         est_sp = avg_sp;
      %END;
   %ELSE %IF &s_est_stat = SUM %THEN
      %DO;
         est_sp = sz_sp    * avg_sp;
         var_sp = sz_sp**2 * var_sp;
      %END;
   LABEL est_sp = "Estd subp &s_su_attr &s_est_stat"
         var_sp = "Estd var of subp &s_su_attr &s_est_stat est";
RUN;

***************************** STEP 4 **************************************;
*STEP 4 - Calculate weighted subpop contribs to pop estimates;
TITLE3 "Weighted subpop contributions to pop estimates";
DATA WORK.m_su4 (LABEL = "Subp contrib to pop est");
   MERGE WORK.m_su3
         WORK.m_sp(KEEP = sp wt_sp);
   BY sp;
   %IF &s_est_stat = MEAN %THEN
      %DO;
         w_est_sp = wt_sp    * est_sp;
         w_var_sp = wt_sp**2 * var_sp;
      %END;
   %ELSE %IF &s_est_stat = SUM %THEN
      %DO;
         w_est_sp =  est_sp;
         w_var_sp =  var_sp;
      %END;
   LABEL w_est_sp = "Wtd estd subp &s_su_attr &s_est_stat"
         w_var_sp = "Wtd estd var of subp &s_su_attr &s_est_stat est";
RUN;

******************************* STEP 5 ***********************************;
*STEP 5 - Sum subpop contribs to get population parameter estimates;
PROC SORT DATA = WORK.m_su4;
   BY &s_su_rows 
      &s_su_cols;
RUN;
PROC SUMMARY DATA = WORK.m_su4;
   VAR w_est_sp w_var_sp;
   BY  &s_su_rows 
       &s_su_cols;
   OUTPUT OUT = WORK.m_final (LABEL = "Pop estimates"
                              DROP = _TYPE_ _FREQ_)
          SUM = &s_est_stat  v_&s_est_stat;
RUN;
DATA WORK.m_final;
   SET WORK.m_final;
   sd_&s_est_stat = SQRT(v_&s_est_stat);
   LABEL &s_est_stat    = "Pop &s_su_attr &s_est_stat est"
         v_&s_est_stat  = "Estd var of pop &s_su_attr &s_est_stat est"
         sd_&s_est_stat = "Estd std of pop &s_su_attr &s_est_stat est";
RUN;

TITLE3 "Final population estimates";
TITLE4 "STRAT macro step 5";
PROC PRINT DATA=WORK.m_final;
RUN;

PROC DATASETS LIBRARY = work;
    delete m_su1
           m_su2
           m_su3
           m_su4
           m_sp
           m_st
           m_temp;
    *Data SET M_FINAL is preserved for passing DATA to REPOUT module;
RUN;

%PUT Macro STRAT ended;

%MEND strat;

*** END OF MACRO ***;
*** END OF MACRO ***;

%*The following macro invocation is used for development testing;
%*strat( s_est_stat =,
	s_st       =,
	s_st_spid  =,
	s_st_stid  =,
	s_st_ssz   =,
	s_st_sz    =,
	s_su       =,
	s_su_suid  =,
	s_su_stid  =,
	s_su_rows  =,
	s_su_cols  =,
	s_su_attr  =,
	s_unit     =,
	s_margins  =);

