/****************************** HEADER *********************************
Larry Bednar, US Forest Service, Portland Forestry Sciences Lab,
         Portland, OR. (503) 808-2016.
PURPOSE: This SAS program calculates population estimates for each cell
         in a table using a simple stratified sampling approach.

         An estimator is used which assumes each cell of the final
         table of estimates to represent summary statistics for ALL
         secondary sample units in the population.  Cell estimates
         represent the population estimate of a variable which takes
         zero values for secondary sample units which do not satisfy
         the conditions indicated by row and column categories in the
         table.

         At present, the program assumes two input data sets, one
         containing observations at the sample unit level, and one
         containing summary statistics for strata.

$RCSfile: strat_rs.sas,v $; 
$Revision: 1.15 $; 
$Date: 2002/02/20 01:45:30 $;   
$Author: lbednar $; 
***********************************************************************/

%macro strat_rs(srs_est_stat      =,
                srs_st            =,
	        srs_st_spid       =,
	        srs_st_stid       =,
	        srs_st_ssz        =,
	        srs_st_sz         =,
	        srs_su            =,
	        srs_su_suid       =,
	        srs_su_stid       =,
	        srs_su_rows       =,
	        srs_su_cols       =,
	        srs_su_attr       =,
                srs_su_attr_denom =,
	        srs_unit          =,
	        srs_margins       =);

%put Macro STRAT_RS started;

%*Change to uppercase to facilitate comparisons;
%LET srs_est_stat = %UPCASE(&srs_est_stat);
%LET srs_margins  = %UPCASE(&srs_margins);

%*Retrieve input data into working DATA sets;
   %*Revision 1.6 eliminates the need for running GET_SP;
   %get_st_s(gss_data_in  = &srs_st,
             gss_sp       = &srs_st_spid,
             gss_id       = &srs_st_stid,
             gss_sz       = &srs_st_sz,
             gss_ssz      = &srs_st_ssz,
             gss_data_out = WORK.stratum);
   %get_ob_s(gos_data_in  = &srs_su,
             gos_st       = &srs_su_stid,
             gos_id       = &srs_su_suid,
             gos_r_vars   = &srs_su_rows,
             gos_c_vars   = &srs_su_cols,
             gos_an       = &srs_su_attr,
             gos_an_denom = &srs_su_attr_denom,
             gos_unit     = &srs_unit,
             gos_data_out = WORK.obs);

*The program assumes existence of input DATA sets WORK.SUBPOP, WORK.STRATA
 and WORK.OBS;

PROC DATASETS LIBRARY = work;
    DELETE m_su1
           m_su2
           m_su3
           m_su4
           m_sp
           m_st
           m_temp
           m_final;
RUN;

*Calculate weights of subpopulations relative to complete target
 population;
PROC SUMMARY DATA = WORK.stratum;
   VAR    sz_h;
   CLASS  sp;
   OUTPUT OUT = WORK.m_sp
          SUM = sz_sp;
RUN;
DATA WORK.m_sp (LABEL = "Subpop attributes");
   SET    WORK.m_sp;
   RETAIN sz_pop 0;
   IF _TYPE_ = 0
      THEN sz_pop = sz_sp;
   ELSE IF _TYPE_ = 1 THEN
      DO;
         wt_sp = sz_sp / sz_pop;
         OUTPUT;
      END;
   DROP _TYPE_ _FREQ_;
RUN;

*Calculate subpop sample sizes;
PROC SORT DATA = WORK.stratum;
    BY sp h;
RUN;
PROC SUMMARY DATA = WORK.stratum;
    VAR    n_h;
    BY     sp;
    OUTPUT OUT = WORK.m_temp (DROP = _TYPE_ _FREQ_)
           SUM = n;
RUN;
DATA WORK.m_sp;
   MERGE WORK.m_temp
         WORK.m_sp;
   BY    sp;
   LABEL sp     ="Subpop ID"
         sz_pop ="Pop size"
         sz_sp  ="Subpop size"
         wt_sp  ="Subpop wt"
         n      ="Subpop samp size";
RUN;
TITLE3 "Subpopulation statistics";
PROC PRINT DATA = WORK.m_sp;
   VAR sp sz_pop sz_sp wt_sp n;
RUN;

*Calculate subpopulation stratum wts;
DATA WORK.m_st(LABEL = "Stratum attributes");
   MERGE WORK.m_sp
         WORK.stratum;
   BY    sp;
   w_h = sz_h / sz_sp;
   f_h = n_h  / sz_h;
   IF h > .Z THEN OUTPUT;
   LABEL w_h = "Subpop stratum wt"
         f_h = "Subpop stratum samp frac";
   KEEP  sp h sz_sp sz_h w_h n n_h f_h;
RUN;
TITLE3 "Subpop stratum statistics";
TITLE4 "STRAT_RS macro step 0";
PROC PRINT DATA = WORK.m_st
           uniform;
   VAR sp sz_sp n h sz_h w_h n_h f_h;
   SUM                   w_h n_h;
RUN;

*Preparation to calculate secondary sample summary statistics for strata;
*Match sample unit observations to stratum statistics;
PROC SORT DATA = WORK.m_st;
   BY h;
RUN;
PROC SORT DATA = WORK.obs;
   BY h;
RUN;
DATA WORK.m_su(LABEL = "Merged stratum/su attributes");
   MERGE WORK.obs  (KEEP = h su_id &srs_su_rows &srs_su_cols 
                           attrib attrib2
                    IN   = obs_in)
         WORK.m_st (KEEP = sp h);
   BY    h;
   IF obs_in = 1 THEN OUTPUT;
RUN;

*Sum any subcategorizations of sample unit observations within
 input observations to level of specified response category variables;
PROC SORT DATA=WORK.m_su;
   BY sp 
      h 
      su_id 
      &srs_su_cols 
      &srs_su_rows;
RUN;
PROC SUMMARY DATA=WORK.m_su;
   VAR attrib
       attrib2;
   %IF &srs_margins = YES %THEN
      %DO;
         *These statments result in generation of estimates for
          marginal populations;
         CLASS &srs_su_cols &srs_su_rows;
         BY sp 
            h 
            su_id;
      %END;
   %ELSE
      %DO;
         *This statement results in generation of estimates for
          only those response categories provided in the input
          observation data set;
         BY sp 
            h 
            su_id 
            &srs_su_cols 
            &srs_su_rows;
      %END;
   OUTPUT OUT = WORK.m_su (DROP=_TYPE_ _FREQ_)
          SUM = y
                x;
RUN;
DATA WORK.m_su;
   SET WORK.m_su;
   yx = y * x;
RUN;

******************************* STEP 1 ************************************;
*STEP 1 - Calculate stratum sample sums, uncorr SSs, covariance yx;
*Within-sample-unit sums of response categories;
*Statum-level sums - USSs and SUMs;
PROC SORT DATA = WORK.m_su;
   BY sp 
      h 
      &srs_su_rows 
      &srs_su_cols;
RUN;
PROC SUMMARY DATA = WORK.m_su;
   VAR    y
          x
          yx;
   BY     sp 
          h 
          &srs_su_rows 
          &srs_su_cols;
   OUTPUT OUT = WORK.m_su1 (DROP = _TYPE_ _FREQ_ yx_uss_h)
          USS = y_uss_h
                x_uss_h
                yx_uss_h
          SUM = y_sum_h
                x_sum_h
                yx_sum_h;
RUN;
DATA WORK.m_su1 (LABEL = "Stratum samp sums");
   SET   WORK.m_su1;
   LABEL x_uss_h  = "Uncorr var Y stratum SS"
         y_uss_h  = "Uncorr var X stratum SS"
         x_sum_h  = "Analysis var X stratum sum"
         y_sum_h  = "Analysis var Y stratum sum"
         yx_sum_h = "Analysis var YX stratum sum";
RUN;
   
   *Change blank values of row and col variables to "ALL";
   %chg_blnk(data_in  = WORK.m_su1,
             chg_vars = &srs_su_rows &srs_su_cols,
             new_val  = ALL,
             data_out = WORK.m_su1);


   TITLE3 "Subpop stratum statistics";
   TITLE4 "STRAT_RS macro step 1";
   PROC PRINT DATA = WORK.m_su1;
   RUN;

******************************* STEP 2 *********************************;
*STEP 2 - Calculate stratum contributions to subpop estimates;
PROC SORT DATA = WORK.m_su1;
    BY sp 
       h;
RUN;
PROC SORT DATA = WORK.m_st;
    BY sp 
       h;
RUN;
DATA WORK.m_su2 (LABEL = "Stratum samp mean/var");
    MERGE WORK.m_st
          WORK.m_su1;
    BY    sp 
          h;
    y_avg_h = y_sum_h / n_h;
    x_avg_h = x_sum_h / n_h;
    IF n_h > 1 THEN 
       DO;
          y_s2_h = (y_uss_h - n_h*(y_avg_h**2) )/(n_h-1);
          x_s2_h = (x_uss_h - n_h*(x_avg_h**2) )/(n_h-1);
       END;
    ELSE IF n_h = 1 THEN 
       DO; 
          y_s2_h = 0;
          x_s2_h = 0;
       END;
    ELSE 
       DO;
          y_s2_h = .c;
          x_s2_h = .c;
       END;
    r_h       = y_sum_h / x_sum_h;
    w_r_h     = w_h    * r_h;
    cov_yx_h  = (yx_sum_h - n_h*y_avg_h*x_avg_h) / (n_h - 1);
    var_r_h   =   ( (1 - f_h) / ( n_h * x_avg_h**2 ) )
                 *( y_s2_h + (r_h**2)*(x_s2_h) - 2*r_h*cov_yx_h );
    w_var_r_h = w_h**2 * var_r_h;
    w_x_avg_h  = w_h * x_avg_h;
    w_y_avg_h  = w_h * y_avg_h;
    var_x_h    = ( 1 - f_h ) * x_s2_h / n_h;
    var_y_h    = ( 1 - f_h ) * y_s2_h / n_h;
    w_var_x_h  = w_h**2 * var_x_h;
    w_var_y_h  = w_h**2 * var_y_h;
    w_cov_yx_h =   (1/n)
                 * (   ( 1 - n/sz_sp) * w_h * (cov_yx_h / n_h)
                     + ( 1 - w_h ) * cov_yx_h / (n * n_h));
    LABEL w_cov_yx_h = "Wtd cell contrib to subpop cov yx"
          w_var_r_h  = "Wtd cell stratum var r"
          w_x_avg_h  = "Wtd cell var X stratum mean"
          w_y_avg_h  = "Wtd cell var Y stratum mean"
          w_var_x_h  = "Wtd cell var X stratum std dev"
          w_var_y_h  = "Wtd cell var Y stratum std dev"
          var_r_h    = "Estd var of stratum ratio est"
          x_avg_h    = "Cell var X stratum mean"
          x_sum_h    = "Cell var X stratum SS"
          x_s2_h     = "Cell var X stratum sample var"
          x_uss_h    = "Cell var X stratum uncorr SS"
          y_avg_h    = "Cell var Y stratum mean"
          y_sum_h    = "Cell var Y stratum SS"
          y_s2_h     = "Cell var Y stratum sample var"
          y_uss_h    = "Cell var Y stratum uncorr SS";
RUN;
PROC SORT DATA = WORK.m_su2;
    BY sp
       &srs_su_rows 
       &srs_su_cols
       h;
RUN;
TITLE3 "Stratum sample X and Y statistics";
TITLE4 "STRAT_RS macro step 2";
PROC PRINT DATA = WORK.m_su2;
    VAR sp  &srs_su_rows &srs_su_cols h n_h y_sum_h   x_sum_h 
                                            y_uss_h   x_uss_h 
                                            y_avg_h   x_avg_h 
                                            y_s2_h    x_s2_h  
                                            yx_sum_h  cov_yx_h;
    BY  sp;
RUN;
TITLE3 "Stratum sample ratio statistics";
TITLE4 "STRAT_RS macro step 2";
PROC PRINT DATA = WORK.m_su2;
    VAR sp &srs_su_rows &srs_su_cols h w_h r_h      var_r_h 
                                           w_r_h    w_var_r_h;
    BY  sp;
RUN;

******************************** STEP 3 ***********************************;
*STEP 3 - Calculate subpop estimates by adding wtd stratum contributions;
PROC SORT DATA = WORK.m_su2;
    BY sp 
       &srs_su_rows 
       &srs_su_cols;
RUN;
PROC SUMMARY DATA = WORK.m_su2;
   VAR w_r_h 
       w_var_r_h
       w_x_avg_h
       w_y_avg_h
       w_var_x_h
       w_var_y_h
       w_cov_yx_h;
   BY  sp 
       &srs_su_rows 
       &srs_su_cols;
   OUTPUT OUT = WORK.m_su3 (LABEL = "Subpop estd mean"
                       DROP = _TYPE_ _FREQ_)
          SUM = rs_sp
                var_rs_sp 
                x_avg_sp
                y_avg_sp
                var_x_sp
                var_y_sp
                cov_yx_sp;
RUN;

DATA WORK.m_su3     (LABEL = "Subpop parameter est");
   SET WORK.m_su3;
   rc_sp     = y_avg_sp / x_avg_sp;
   var_rc_sp =   ( 1 / x_avg_sp**2 )
               * ( var_y_sp + rc_sp**2 * var_x_sp - 2 * cov_yx_sp );
   LABEL cov_yx_sp = "Estd subp cov Y,X"
         rc_sp     = "Estd subp rom_c"
         var_rc_sp = "Estd subp rom_c var"
         rs_sp     = "Estd subp rom_s"
         var_rs_sp = "Estd var of rom_s"
         x_avg_sp  = "Estd subp su mean X"
         y_avg_sp  = "Estd subp su mean Y"
         var_x_sp  = "Estd var of subp su mean X"
         var_y_sp  = "Estd var of subp su mean Y";
RUN;

TITLE3 "Subpopulation statistics";
TITLE4 "STRAT_RS macro step 3";
PROC PRINT DATA=WORK.m_su3;
RUN;

***************************** STEP 4 **************************************;
*STEP 4 - Calculate weighted subpop contribs to pop estimates;
TITLE3 "Weighted subpop contributions to pop estimates";
DATA WORK.m_su4 (LABEL = "Subp contrib to pop est");
   MERGE WORK.m_su3
         WORK.m_sp (KEEP = sp wt_sp);
   BY sp;
   w_rc_sp     = wt_sp    * rc_sp;
   w_var_rc_sp = wt_sp**2 * var_rc_sp;
   w_rs_sp     = wt_sp    * rs_sp;
   w_var_rs_sp = wt_sp**2 * var_rs_sp;
   w_x_sp      = wt_sp    * x_avg_sp;
   w_var_x_sp  = wt_sp**2 * var_x_sp;
   w_y_sp      = wt_sp    * y_avg_sp;
   w_var_y_sp  = wt_sp**2 * var_y_sp;
   LABEL w_rc_sp     = "Wtd estd subp rom_c est"
         w_var_rc_sp = "Wtd estd var of subp rom_c"
         w_rs_sp     = "Wtd estd subp rom_s est"
         w_var_rs_sp = "Wtd estd var of subp rom_s";
RUN;

******************************* STEP 5 ***********************************;
*STEP 5 - Sum subpop contribs to get population parameter estimates;
PROC SORT DATA = WORK.m_su4;
   BY &srs_su_rows 
      &srs_su_cols;
RUN;
PROC SUMMARY DATA = WORK.m_su4;
   %IF &srs_est_stat = ROM_S %THEN
      %DO;
         VAR w_rs_sp 
             w_var_rs_sp;
      %END;
   %IF &srs_est_stat = ROM_C %THEN
      %DO;
         VAR w_rc_sp
             w_var_rc_sp;
      %END;
   BY  &srs_su_rows 
       &srs_su_cols;
   OUTPUT OUT = WORK.m_final (LABEL = "Pop estimates"
                              DROP = _TYPE_ _FREQ_)
          SUM = &srs_est_stat  
                v_&srs_est_stat;
RUN;
DATA WORK.m_final;
   SET WORK.m_final;
   sd_&srs_est_stat = SQRT(v_&srs_est_stat);
   LABEL &srs_est_stat    = "Estd pop &srs_su_attr/&srs_su_attr_denom &srs_est_stat"
         v_&srs_est_stat  = "Estd var of pop &srs_su_attr/&srs_su_attr &srs_est_stat"
         sd_&srs_est_stat = "Estd std of pop &srs_su_attr/&srs_su_attr &srs_est_stat";
RUN;

TITLE3 "Final population estimates";
TITLE4 "STRAT_RS macro step 5";
PROC PRINT DATA=WORK.m_final;
   VAR &srs_su_rows &srs_su_cols &srs_est_stat 
                                 v_&srs_est_stat 
                                 sd_&srs_est_stat;
RUN;

PROC DATASETS LIBRARY = work;
    DELETE m_cov
           m_su1
           m_su2
           m_su3
           m_su4
           m_su_r
           m_sp
           m_st
           m_temp;
    *Data SET M_FINAL is preserved for passing DATA to REPOUT module;
RUN;

%PUT Macro STRAT_RS ended;

%MEND strat_rs;

*** END OF MACRO ***;
*** END OF MACRO ***;

%*The following macro invocation is used for development testing;
%*strat_rs( srs_est_stat      =,
           srs_st            =,
	   srs_st_spid       =,
	   srs_st_stid       =,
	   srs_st_ssz        =,
	   srs_st_sz         =,
	   srs_su            =,
	   srs_su_suid       =,
	   srs_su_stid       =,
	   srs_su_rows       =,
	   srs_su_cols       =,
	   srs_su_attr       =,
           srs_su_attr_denom =,
	   srs_unit          =,
	   srs_margins       =);

